﻿/*
This file is part of My Nes
A Nintendo Entertainment System Emulator.

 Copyright © 2009 - 2010 Ala Hadid (AHD)

My Nes is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

My Nes is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace MyNes
{
    public class MFolder
    {
        string _Name = "";
        string _Path = "";
        int _Mapper = 0;
        string _ImagesFolder = "";
        string _InfosFolder = "";
        FolderFilter _Filter = FolderFilter.SupportedMappersOnly;
        public string Name
        {
            get { return _Name; }
            set
            {
                _Name = value;
                if (NameChanged != null)
                { NameChanged(this, null); }
            }
        }
        public string Path
        { get { return _Path; } set { _Path = value; } }
        public string ImagesFolder
        { get { return _ImagesFolder; } set { _ImagesFolder = value; } }
        public string InfosFolder
        { get { return _InfosFolder; } set { _InfosFolder = value; } }
        public FolderFilter Filter
        { get { return _Filter; } set { _Filter = value; } }
        public int Mapper
        { get { return _Mapper; } set { _Mapper = value; } }
        public event EventHandler<EventArgs> NameChanged;
    }
    public class TreeNode_Folder : TreeNode
    {
        MFolder _Folder = new MFolder();
        public TreeNode_Folder()
        {
            _Folder.NameChanged += new EventHandler<EventArgs>(_Folder_NameChanged);
        }
        void _Folder_NameChanged(object sender, EventArgs e)
        {
            this.Text = _Folder.Name;
        }
        public MFolder Folder
        {
            get { return _Folder; }
            set
            {
                _Folder = value;
                this.Text = _Folder.Name;
                Nodes.Clear();
                string[] dirs = Directory.GetDirectories(value.Path);
                foreach (string dir in dirs)
                {
                    MFolder fol = new MFolder();
                    fol.Path = dir;
                    fol.Name = Path.GetFileName( dir);
                    TreeNode_Folder TR = new TreeNode_Folder();
                    TR.Folder = fol;
                    TR.ImageIndex = this.ImageIndex;
                    TR.SelectedImageIndex = this.SelectedImageIndex;
                    Nodes.Add(TR);
                }
            }
        }
    }
    public class ListViewItem_Rom : ListViewItem
    {
        string _RomPath;
        public string RomPath
        { get { return _RomPath; } set { _RomPath = value; } }
    }
    public class MFolderCollection : List<MFolder>
    {

    }
    public enum FolderFilter
    { All, SupportedMappersOnly, Mapper }
}
