﻿/*
This file is part of My Nes
A Nintendo Entertainment System Emulator.

 Copyright © 2009 - 2010 Ala Hadid (AHD)

My Nes is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

My Nes is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using MyNes.Nes;
using System.IO;
using System.Reflection;
using System.Diagnostics;

namespace MyNes
{
    static class Program
    {
        static Properties.Settings _Settings;
        static Frm_Main _MainForm;
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        /// <param name="Args">The command lines</param>
        [STAThread]
        static void Main(string[] Args)
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            //Let's see if My Nes is running or not
            foreach (Process PR in Process.GetProcesses())
            {
                if (PR.MainWindowTitle.Length >= "My Nes".Length)
                {
                    if (PR.MainWindowTitle.Substring(0, "My Nes".Length) == "My Nes" & PR != Process.GetCurrentProcess())
                    {
                        PR.CloseMainWindow();//Kill the old one
                        break;
                    }
                }
            }
            //Load settings
            _Settings = new MyNes.Properties.Settings();
            _Settings.Reload();
            //Load the main window
            _MainForm = new Frm_Main(Args);
            _MainForm.LoadSettings();
            _MainForm.ShowDialogs();
            Application.Run(_MainForm);
        }
        public static Properties.Settings Settings
        { get { return _Settings; } }
        public static Frm_Main Form_Main
        { get { return _MainForm; } set { _MainForm = value; } }
        /// <summary>
        /// Get file size
        /// </summary>
        /// <param name="FilePath">Full path of the file</param>
        /// <returns>Return file size + unit lable</returns>
        public static string GetFileSize(string FilePath)
        {
            if (File.Exists(Path.GetFullPath(FilePath)) == true)
            {
                FileInfo Info = new FileInfo(FilePath);
                string Unit = " Byte";
                double Len = Info.Length;
                if (Info.Length >= 1024)
                {
                    Len = Info.Length / 1024.00;
                    Unit = " KB";
                }
                if (Len >= 1024)
                {
                    Len /= 1024.00;
                    Unit = " MB";
                }
                if (Len >= 1024)
                {
                    Len /= 1024.00;
                    Unit = " GB";
                }
                return Len.ToString("F2") + Unit;
            }
            return "";
        }
        /// <summary>
        /// Get the version
        /// </summary>
        public static string AssemblyVersion
        {
            get
            {
                return Assembly.GetExecutingAssembly().GetName().Version.ToString();
            }
        }
    }
}
