﻿/*
This file is part of My Nes
A Nintendo Entertainment System Emulator.

 Copyright © 2009 - 2010 Ala Hadid (AHD)

My Nes is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

My Nes is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using MyNes.Nes;

namespace MyNes
{
    public partial class Frm_Browser : Form
    {
        Frm_Main _TheMainForm;
        TreeNode_Folder _SelectedFolder;
        public Frm_Browser(Frm_Main TheMainForm)
        {
            _TheMainForm = TheMainForm;
            InitializeComponent();
            //Load settings
            checkBox1.Checked = Program.Settings.ShowBrowser;
            this.Location = new Point(Program.Settings.Browser_X, Program.Settings.Browser_Y);
            this.Size = new Size(Program.Settings.Browser_W, Program.Settings.Browser_H);
            splitContainer1.SplitterDistance = Program.Settings.Container1;
            splitContainer2.SplitterDistance = Program.Settings.Container2;
            columnHeader1_Name.Width = Program.Settings.Column_Name;
            columnHeader2_Size.Width = Program.Settings.Column_Size;
            columnHeader3_Mapper.Width = Program.Settings.Column_Mapper;
            listView1.View = Program.Settings.View;
            switch (Program.Settings.View)
            {
                case View.Details:
                    tilesToolStripMenuItem.Checked = false;
                    detailsToolStripMenuItem.Checked = true;
                    listToolStripMenuItem.Checked = false;
                    break;
                case View.Tile:
                    tilesToolStripMenuItem.Checked = true;
                    detailsToolStripMenuItem.Checked = false;
                    listToolStripMenuItem.Checked = false;
                    break;
                case View.List:
                    tilesToolStripMenuItem.Checked = false;
                    detailsToolStripMenuItem.Checked = false;
                    listToolStripMenuItem.Checked = true;
                    break;
            }
            //Load folders
            if (Program.Settings.Folders == null)
                Program.Settings.Folders = new MFolderCollection();
            RefreshFolders();
        }
        void RefreshFolders()
        {
            treeView1.Nodes.Clear();
            foreach (MFolder fol in Program.Settings.Folders)
            {
                TreeNode_Folder TR = new TreeNode_Folder();
                TR.ImageIndex = 0;
                TR.SelectedImageIndex = 1;
                TR.Folder = fol;
                treeView1.Nodes.Add(TR);
            }
        }
        void SaveSettings()
        {
            Program.Settings.ShowBrowser = checkBox1.Checked;
            Program.Settings.Browser_X = this.Location.X;
            Program.Settings.Browser_Y = this.Location.Y;
            Program.Settings.Browser_H = this.Height;
            Program.Settings.Browser_W = this.Width;
            Program.Settings.Container1 = splitContainer1.SplitterDistance;
            Program.Settings.Container2 = splitContainer2.SplitterDistance;
            Program.Settings.Column_Name = columnHeader1_Name.Width;
            Program.Settings.Column_Size = columnHeader2_Size.Width;
            Program.Settings.Column_Mapper = columnHeader3_Mapper.Width;
            Program.Settings.Save();
        }
        private void button2_Click(object sender, EventArgs e)
        {
            SaveSettings();
            Close();
        }
        private void AddFolder(object sender, EventArgs e)
        {
            FolderBrowserDialog fo = new FolderBrowserDialog();
            fo.Description = "Add roms folder";
            fo.ShowNewFolderButton = true;
            if (fo.ShowDialog(this) == DialogResult.OK)
            {
                MFolder folderr = new MFolder();
                folderr.Path = fo.SelectedPath;
                folderr.Name = Path.GetFileName(fo.SelectedPath);
                Program.Settings.Folders.Add(folderr);
                RefreshFolders();
            }
        }
        private void DeleteFolder(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;
            if (treeView1.SelectedNode.Parent != null)
                return;
            if (MessageBox.Show("Are you sure you want to delete selected folder from the list ?", "Delete folder", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                Program.Settings.Folders.RemoveAt(treeView1.SelectedNode.Index);
                RefreshFolders();
            }
        }
        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;
            listView1.Items.Clear();
            pictureBox1.Image = Properties.Resources.NESDoc;
            richTextBox1.Text = "";
            TreeNode_Folder TR = (TreeNode_Folder)treeView1.SelectedNode;
            _SelectedFolder = TR;
            TextBox1_mapper.Text = _SelectedFolder.Folder.Mapper.ToString();
            switch (_SelectedFolder.Folder.Filter)
            {
                case FolderFilter.All: ComboBox1_nav.SelectedIndex = 0; TextBox1_mapper.Enabled = false; break;
                case FolderFilter.SupportedMappersOnly: ComboBox1_nav.SelectedIndex = 1; TextBox1_mapper.Enabled = false; break;
                case FolderFilter.Mapper: ComboBox1_nav.SelectedIndex = 2; TextBox1_mapper.Enabled = true; break;
            }
            if (!Directory.Exists(TR.Folder.Path))
            {
                MessageBox.Show("This folder isn't exist on the disk !!");
                DeleteFolder(this, null);
            }
            else
            {
                string[] Dirs = Directory.GetFiles(TR.Folder.Path);
                foreach (string Dir in Dirs)
                {
                    string EXT = Path.GetExtension(Dir);
                    switch (EXT.ToLower())
                    {
                        case ".nes":
                            ListViewItem_Rom IT = new ListViewItem_Rom();
                            IT.RomPath = Dir;
                            IT.ImageIndex = 2;
                            IT.Text = Path.GetFileName(Dir);
                            IT.SubItems.Add(Program.GetFileSize(Dir));
                            INESHeaderReader header = new INESHeaderReader(Dir);
                            if (header.validRom)
                                IT.SubItems.Add(header.MemoryMapper.ToString() + ", " + header.GetMapperName());
                            switch (TR.Folder.Filter)
                            {
                                case FolderFilter.All: listView1.Items.Add(IT); break;
                                case FolderFilter.SupportedMappersOnly:
                                    if (header.SupportedMapper())
                                        listView1.Items.Add(IT);
                                    break;
                                case FolderFilter.Mapper:
                                    if (header.MemoryMapper == Convert.ToInt32(TextBox1_mapper.Text))
                                        listView1.Items.Add(IT);
                                    break;
                            }
                            break;
                        case ".rar":
                            ListViewItem_Rom IT1 = new ListViewItem_Rom();
                            IT1.ImageIndex = 3;
                            IT1.RomPath = Dir;
                            IT1.Text = Path.GetFileName(Dir);
                            IT1.SubItems.Add(Program.GetFileSize(Dir));
                            IT1.SubItems.Add("N/A");
                            listView1.Items.Add(IT1);
                            break;
                        case ".zip":
                            ListViewItem_Rom IT2 = new ListViewItem_Rom();
                            IT2.ImageIndex = 3;
                            IT2.RomPath = Dir;
                            IT2.Text = Path.GetFileName(Dir);
                            IT2.SubItems.Add(Program.GetFileSize(Dir));
                            IT2.SubItems.Add("N/A");
                            listView1.Items.Add(IT2);
                            break;
                        case ".7z":
                            ListViewItem_Rom IT3 = new ListViewItem_Rom();
                            IT3.ImageIndex = 3;
                            IT3.RomPath = Dir;
                            IT3.Text = Path.GetFileName(Dir);
                            IT3.SubItems.Add(Program.GetFileSize(Dir));
                            IT3.SubItems.Add("N/A");
                            listView1.Items.Add(IT3);
                            break;
                    }
                }
                label1_status.Text = listView1.Items.Count.ToString() + " items found.";
            }
        }
        private void RenameSelected(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;
            if (treeView1.SelectedNode.Parent != null)
                return;
            treeView1.SelectedNode.BeginEdit();
        }
        private void treeView1_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            if (treeView1.SelectedNode == null)
            { e.CancelEdit = true; return; }
            if (treeView1.SelectedNode.Parent != null)
            { e.CancelEdit = true; return; }
            if (e.Label == null)
            { return; }
            TreeNode_Folder fol = (TreeNode_Folder)treeView1.SelectedNode;
            fol.Folder.Name = e.Label;
            fol.Text = e.Label;
        }
        //Play
        private void PlayRom(object sender, MouseEventArgs e)
        {
            if (listView1.SelectedItems.Count != 1)
                return;
            ListViewItem_Rom IT = (ListViewItem_Rom)listView1.SelectedItems[0];
            if (File.Exists(IT.RomPath))
            {
                SaveSettings();
                _TheMainForm.OpenRom(IT.RomPath);
                _TheMainForm.Select();
            }
        }
        private void listView1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
                PlayRom(this, null);
        }
        private void Frm_Browser_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
                PlayRom(this, null);
        }
        private void button1_Click(object sender, EventArgs e)
        {
            PlayRom(this, null);
        }
        private void ChangeImageFolder(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;
            if (treeView1.SelectedNode.Parent != null)
                return;
            TreeNode_Folder TR = (TreeNode_Folder)treeView1.SelectedNode;
            FolderBrowserDialog Fol = new FolderBrowserDialog();
            Fol.Description = "Images folder";
            Fol.ShowNewFolderButton = true;
            Fol.SelectedPath = TR.Folder.ImagesFolder;
            if (Fol.ShowDialog() == DialogResult.OK)
            {
                TR.Folder.ImagesFolder = Fol.SelectedPath;
            }
        }
        private void toolStripButton7_Click(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;
            if (treeView1.SelectedNode.Parent != null)
                return;
            TreeNode_Folder TR = (TreeNode_Folder)treeView1.SelectedNode;
            FolderBrowserDialog Fol = new FolderBrowserDialog();
            Fol.Description = "Info files folder";
            Fol.ShowNewFolderButton = true;
            Fol.SelectedPath = TR.Folder.InfosFolder;
            if (Fol.ShowDialog() == DialogResult.OK)
            {
                TR.Folder.InfosFolder = Fol.SelectedPath;
            }
        }
        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox1.Image = Properties.Resources.NESDoc;
            richTextBox1.Text = "";
            if (listView1.SelectedItems.Count != 1)
                return;
            if (treeView1.SelectedNode == null)
                return;
            if (_SelectedFolder == null)
                return;
            #region Load Image
            try
            {
                string[] Exs = { ".jpg", ".JPG", ".png", ".PNG", ".bmp", ".BMP", ".gif", ".GIF", ".jpeg", ".JPEG", ".tiff", ".TIFF" };
                string[] Names = Directory.GetFiles(_SelectedFolder.Folder.ImagesFolder);
                string FileName = Path.GetFileNameWithoutExtension(listView1.SelectedItems[0].Text);
                foreach (string STR in Names)
                {
                    if (Path.GetFileNameWithoutExtension(STR).Length >= Path.GetFileNameWithoutExtension(listView1.SelectedItems[0].Text).Length)
                    {
                        if (Path.GetFileNameWithoutExtension(STR).Substring(0, Path.GetFileNameWithoutExtension(listView1.SelectedItems[0].Text).Length) == Path.GetFileNameWithoutExtension(listView1.SelectedItems[0].Text))
                        { FileName = Path.GetFileNameWithoutExtension(STR); break; }
                    }
                }
                foreach (string STR in Exs)
                {
                    if (File.Exists(_SelectedFolder.Folder.ImagesFolder + "//" +
                        FileName + STR))
                    {
                        pictureBox1.Image = Image.FromFile(_SelectedFolder.Folder.ImagesFolder + "//" +
                        FileName + STR); break;
                    } pictureBox1.Image = Properties.Resources.NESDoc;
                }
            }
            catch { pictureBox1.Image = Properties.Resources.NESDoc; }
            #endregion
            #region Load Info Text
            try
            {
                string FileName = Path.GetFileNameWithoutExtension(listView1.SelectedItems[0].Text);
                richTextBox1.Text = ""; //Clear
                richTextBox1.Lines = File.ReadAllLines(_SelectedFolder.Folder.InfosFolder + "//" + FileName + ".txt");
            }
            catch
            {
                richTextBox1.Text = "";
            }
            #endregion
        }
        private void Frm_Browser_FormClosed(object sender, FormClosedEventArgs e)
        {
            SaveSettings();
        }
        private void toolStripButton4_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count != 1)
                return;
            if (treeView1.SelectedNode == null)
                return;
            if (_SelectedFolder == null)
                return;
            string FileName = Path.GetFileNameWithoutExtension(listView1.SelectedItems[0].Text);
            if (Directory.Exists(_SelectedFolder.Folder.InfosFolder))
            {
                File.WriteAllLines(_SelectedFolder.Folder.InfosFolder + "//" + FileName
                    + ".txt", richTextBox1.Lines, Encoding.Unicode);
            }
            else
            {
                if (MessageBox.Show("This folder hasn't a folder for info files, do you want to browse for it now ?", "No Infos Folder !!", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    FolderBrowserDialog Fol = new FolderBrowserDialog();
                    Fol.Description = "Info files folder";
                    Fol.ShowNewFolderButton = true;
                    Fol.SelectedPath = _SelectedFolder.Folder.InfosFolder;
                    if (Fol.ShowDialog() == DialogResult.OK)
                    {
                        _SelectedFolder.Folder.InfosFolder = Fol.SelectedPath;
                        File.WriteAllLines(_SelectedFolder.Folder.InfosFolder + "//" + FileName
                   + ".txt", richTextBox1.Lines, Encoding.Unicode);
                    }
                }
            }
        }
        private void toolStripButton5_Click(object sender, EventArgs e)
        {
            try
            {
                if (listView1.SelectedItems.Count != 1)
                    return;
                if (treeView1.SelectedNode == null)
                    return;
                if (_SelectedFolder == null)
                    return;
                string FileName = Path.GetFileNameWithoutExtension(listView1.SelectedItems[0].Text);
                richTextBox1.Text = ""; //Clear
                richTextBox1.Lines = File.ReadAllLines(_SelectedFolder.Folder.InfosFolder + "//" + FileName + ".txt");
            }
            catch
            {
                richTextBox1.Text = "";
            }
        }
        private void TextBox1_mapper_Click(object sender, EventArgs e)
        {
            try
            {
                Convert.ToInt32(TextBox1_mapper.Text);
                if (Convert.ToInt32(TextBox1_mapper.Text) > 255)
                    TextBox1_mapper.Text = "255";
                if (_SelectedFolder != null)
                    _SelectedFolder.Folder.Mapper = Convert.ToInt32(TextBox1_mapper.Text);
            }
            catch { TextBox1_mapper.Text = "0"; }
        }
        private void TextBox1_mapper_TextChanged(object sender, EventArgs e)
        {
            try
            {
                Convert.ToInt32(TextBox1_mapper.Text);
                if (Convert.ToInt32(TextBox1_mapper.Text) > 255)
                    TextBox1_mapper.Text = "255";
                if (_SelectedFolder != null)
                    _SelectedFolder.Folder.Mapper = Convert.ToInt32(TextBox1_mapper.Text);
            }
            catch { TextBox1_mapper.Text = "0"; }
        }
        private void toolStripButton8_Click(object sender, EventArgs e)
        {
            treeView1_AfterSelect(this, null);
        }
        private void tilesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.Settings.View = View.LargeIcon;
            tilesToolStripMenuItem.Checked = true;
            detailsToolStripMenuItem.Checked = false;
            listToolStripMenuItem.Checked = false;
            listView1.View = Program.Settings.View;
        }
        private void detailsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.Settings.View = View.Details;
            tilesToolStripMenuItem.Checked = false;
            detailsToolStripMenuItem.Checked = true;
            listToolStripMenuItem.Checked = false;
            listView1.View = Program.Settings.View;
        }
        private void listToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.Settings.View = View.List;
            tilesToolStripMenuItem.Checked = false;
            detailsToolStripMenuItem.Checked = false;
            listToolStripMenuItem.Checked = true;
            listView1.View = Program.Settings.View;
        }
        private void toolStripButton9_Click(object sender, EventArgs e)
        {
            if (listView1.Items.Count == 0)
                return;
            List<ListViewItem_Rom> founded = new List<ListViewItem_Rom>();
            for (int i = 0; i < listView1.Items.Count; i++)
            {
                if (listView1.Items[i].Text.Length >= TextBox1_search.Text.Length&
                    listView1.Items[i].Text.ToLower().Substring(0, TextBox1_search.Text.Length) == TextBox1_search.Text.ToLower())
                {
                    founded.Add((ListViewItem_Rom)listView1.Items[i]);
                }
            }
            listView1.Items.Clear();
            foreach (ListViewItem_Rom innn in founded)
                listView1.Items.Add(innn);
            label1_status.Text = listView1.Items.Count.ToString() + " item(s) matched.";
        }

        private void ComboBox1_nav_DropDownClosed(object sender, EventArgs e)
        {
            if (treeView1.SelectedNode == null)
                return;
            if (_SelectedFolder == null)
                return;
            switch (ComboBox1_nav.SelectedIndex)
            {
                case 0:
                    _SelectedFolder.Folder.Filter = FolderFilter.All;
                    TextBox1_mapper.Enabled = false;
                    break;
                case 1:
                    _SelectedFolder.Folder.Filter = FolderFilter.SupportedMappersOnly;
                    TextBox1_mapper.Enabled = false;
                    break;
                case 2:
                    _SelectedFolder.Folder.Filter = FolderFilter.Mapper;
                    TextBox1_mapper.Enabled = true;
                    break;
            }
            treeView1_AfterSelect(this, null);
        }
    }
}
