﻿/*
This file is part of My Nes
A Nintendo Entertainment System Emulator.

 Copyright © 2009 - 2010 Ala Hadid (AHD)

My Nes is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

My Nes is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MyNes.Nes;

namespace MyNes
{
    public partial class Frm_Debugger : Form
    {
        public Frm_Debugger()
        {
            InitializeComponent();
            this.Location = new Point(Program.Settings.DebuggerWin_X, Program.Settings.DebuggerWin_Y);
            this.Size = new Size(Program.Settings.DebuggerWin_W, Program.Settings.DebuggerWin_H);
            DEBUG.DebugRised += new EventHandler<DebugArg>(DEBUG_DebugRised);
            ClearDebugger();
        }
        void DEBUG_DebugRised(object sender, DebugArg e)
        {
            WriteLine(e.DebugLine, e.Status);
        }
        public void SaveSettings()
        {
            Program.Settings.DebuggerWin_X = this.Location.X;
            Program.Settings.DebuggerWin_Y = this.Location.Y;
            Program.Settings.DebuggerWin_W = this.Width;
            Program.Settings.DebuggerWin_H = this.Height;
            Program.Settings.Save();
        }
        private void Frm_Debugger_FormClosed(object sender, FormClosedEventArgs e)
        {
            SaveSettings();
        }
        private void button3_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        public void WriteLine(string Text, DebugStatus State)
        {
            try
            {
                richTextBox1.SelectionStart = richTextBox1.Text.Length;
                switch (State)
                {
                    case DebugStatus.None:
                        richTextBox1.SelectionColor = Color.White;
                        break;
                    case DebugStatus.Cool:
                        richTextBox1.SelectionColor = Color.LightGreen;
                        break;
                    case DebugStatus.Error:
                        richTextBox1.SelectionColor = Color.Red;
                        break;
                    case DebugStatus.Warning:
                        richTextBox1.SelectionColor = Color.Yellow;
                        break;
                }
                richTextBox1.SelectedText = Text + "\n";
                richTextBox1.SelectionColor = Color.Green;
                richTextBox1.ScrollToCaret();
            }
            catch { }
        }
        public void WriteLine(string Text)
        { WriteLine(Text, DebugStatus.None); }
        public void ClearDebugger()
        {
            richTextBox1.Text = "";
            WriteLine("Welcome to My Nes console");
            WriteLine(@"Write ""Help"" to see a list of instructions.");
            DEBUG.WriteSeparateLine(this, DebugStatus.None);
        }
        void DoInstruction(string Instruction)
        {
            WriteLine(Instruction);
            switch (Instruction.ToLower())
            {
                case "cl d": ClearDebugger(); break;
                case "exit": this.Close(); break;
                case "help": Help(); break;
                case "rom":
                    if (Program.Form_Main.NES != null)
                    {
                        WriteLine("Rom info:", DebugStatus.None);
                        string st =
                        "PRG count: " + Program.Form_Main.NES.MEMORY.MAP.Cartridge.PRG_PAGES.ToString() + "\n" +
                        "CHR count: " + Program.Form_Main.NES.MEMORY.MAP.Cartridge.CHR_PAGES.ToString() + "\n" +
                        "Mapper: #" + Program.Form_Main.NES.MEMORY.MAP.Cartridge.MapperNo.ToString() + "\n";
                        WriteLine(st, DebugStatus.Cool);
                    }
                    else
                    {
                        WriteLine("SYSTEM IS OFF", DebugStatus.Error);
                    }
                    break;
                case "r":
                    WriteLine("Cpu registers:", DebugStatus.None);
                    if (Program.Form_Main.NES != null)
                    {
                        string st =
                            "PC: $" + string.Format("{0:X}", Program.Form_Main.NES.CPU.REG_PC) + "\n"
                            + "A: $" + string.Format("{0:X}", Program.Form_Main.NES.CPU.REG_A) + "\n"
                            + "X: $" + string.Format("{0:X}", Program.Form_Main.NES.CPU.REG_X) + "\n"
                            + "Y: $" + string.Format("{0:X}", Program.Form_Main.NES.CPU.REG_Y) + "\n"
                            + "S: $" + string.Format("{0:X}", Program.Form_Main.NES.CPU.REG_S) + "\n"
                            + "P: $" + string.Format("{0:X}", Program.Form_Main.NES.CPU.StatusRegister());
                        WriteLine(st, DebugStatus.Cool);
                    }
                    else
                    {
                        WriteLine("SYSTEM IS OFF", DebugStatus.Error);
                    }
                    break;
                case "cl sram":
                    if (Program.Form_Main.NES != null)
                    {
                        WriteLine("Clearing the S-RAM ....", DebugStatus.Warning);
                        for (int i = 0; i < Program.Form_Main.NES.MEMORY.SRAM.Length; i++)
                            Program.Form_Main.NES.MEMORY.SRAM[i] = 0;
                        WriteLine("Done.", DebugStatus.Cool);
                    }
                    else
                    {
                        WriteLine("SYSTEM IS OFF", DebugStatus.Error);
                    }
                    break;
                default://Memory instructions handled here
                    if (Instruction == "")
                    { WriteLine("Enter something !!", DebugStatus.Warning); break; }
                    else if (Instruction.ToLower().Substring(0, 2) == "w ")
                    {
                        if (Program.Form_Main.NES == null)
                            WriteLine("SYSTEM IS OFF", DebugStatus.Error);
                        try
                        {
                            ushort ADDRESS = Convert.ToUInt16(Instruction.ToLower().Substring(3, 4), 16);
                            byte VALUE = Convert.ToByte(Instruction.ToLower().Substring(9, 2), 16);
                            Program.Form_Main.NES.MEMORY.Write(ADDRESS, VALUE);
                            WriteLine("Done.", DebugStatus.Cool);
                        }
                        catch { WriteLine("Bad address or value.", DebugStatus.Error); }
                    }
                    else if (Instruction.ToLower().Substring(0, 2) == "m ")
                    {
                        if (Program.Form_Main.NES == null)
                            WriteLine("SYSTEM IS OFF", DebugStatus.Error);
                        try
                        {
                            ushort ADDRESS = Convert.ToUInt16(Instruction.ToLower().Substring(3, 4), 16);
                            byte VALUE = Program.Form_Main.NES.MEMORY.Read(ADDRESS);
                            WriteLine("Done, the value = $" + string.Format("{0:X}", VALUE), DebugStatus.Cool);
                        }
                        catch { WriteLine("Bad address.", DebugStatus.Error); }
                    }
                    else if (Instruction.ToLower().Substring(0, 3) == "mr ")
                    {
                        if (Program.Form_Main.NES == null)
                            WriteLine("SYSTEM IS OFF", DebugStatus.Error);
                        try
                        {
                            ushort ADDRESS = Convert.ToUInt16(Instruction.ToLower().Substring(4, 4), 16);
                            int countt = Convert.ToInt32(Instruction.ToLower().Substring(9, Instruction.ToLower().Length - 9));
                            for (int i = 0; i < countt; i++)
                            {
                                byte VALUE = Program.Form_Main.NES.MEMORY.Read((ushort)(ADDRESS + i));
                                WriteLine("$" + string.Format("{0:X}", (ADDRESS + i)) + " : $" + string.Format("{0:X}", VALUE), DebugStatus.Cool);
                            }
                        }
                        catch { WriteLine("Bad address or range out of memory.", DebugStatus.Error); }
                    }
                    else
                        WriteLine("Bad command ...", DebugStatus.Warning); break;
            }
        }
        void Help()
        {
            WriteLine("Instructions list :", DebugStatus.Cool);
            WriteLine("  > exit : Close the console");
            WriteLine("  > help : Display this list !!");
            WriteLine("  > rom : Show the current rom info.");
            WriteLine("  > R : Dumps the cpu registers");
            WriteLine("  > W <address> <value> : Set a value into the memory at the specific address.");
            WriteLine("  The address and the value must be in hex starting with $");
            WriteLine("  e.g : w $1F23 $10");
            WriteLine("  > M <address> : Dumps the memory at the specific address.");
            WriteLine("  > MR <StartAddress> <Count> : Dumps the specified memory range.");
            WriteLine("  The address must be in hex starting with $, count must be dec");
            WriteLine("  e.g : mr $1F23 1000");
            WriteLine("  > CL D : Clear debugger");
            WriteLine("  > CL SRAM : Clear S-RAM");
            WriteLine("Instructions are NOT case sensitive.");
        }
        private void textBox1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                if (richTextBox1.Lines.Length > 0)
                    richTextBox1.SelectedText = richTextBox1.Lines[richTextBox1.Lines.Length - 1];
                DoInstruction(textBox1.Text);
            }
        }
        private void button2_Click(object sender, EventArgs e)
        {
            if (richTextBox1.Lines.Length > 0)
                richTextBox1.SelectedText = richTextBox1.Lines[richTextBox1.Lines.Length - 1];
            DoInstruction(textBox1.Text);
        }
        private void button1_Click(object sender, EventArgs e)
        {
            ClearDebugger();
        }
    }
}
