﻿/*
This file is part of My Nes
A Nintendo Entertainment System Emulator.

 Copyright © 2009 - 2010 Ala Hadid (AHD)

My Nes is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

My Nes is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace MyNes.Nes
{
    /// <summary>
    /// The 6502
    /// </summary>
    public class CPU
    {
        /// <summary>
        /// The 6502
        /// </summary>
        /// <param name="Memory">The memory</param>
        /// <param name="TvFormat">The TV format</param>
        public CPU(Memory Memory, TVFORMAT TvFormat, NesEngine Nes)
        {
            _Nes = Nes;
            MEM = Memory;
            InitializeCPU(TvFormat);
        }
        //Registers
        public byte REG_A = 0;
        public byte REG_X = 0;
        public byte REG_Y = 0;
        public byte REG_S = 0;
        public ushort REG_PC = 0;
        //Flags
        public bool Flag_N = false;
        public bool Flag_V = false;
        public bool Flag_B = false;
        public bool Flag_D = false;
        public bool Flag_I = true;
        public bool Flag_Z = false;
        public bool Flag_C = false;
        public bool IRQNextTime = false;
        //Clocks and timing stuff
        public int CycleCounter = 0;
        public int CyclesPerScanline = 0;
        //Others
        Memory MEM;
        NesEngine _Nes;
        bool _ON = false;
        bool _Pause = false;
        bool Paused = false;//To ensure the cpu is paused
        public byte OpCode = 0;
        public ushort PrevPC = 0;

        /// <summary>
        /// Run the cpu looping
        /// </summary>
        public void Run()
        {
            PrevPC = REG_PC;
            while (_ON)
            {
                OpCode = MEM.Read(REG_PC);
                if (!_Pause)
                {
                    Paused = false;
                    #region DO OPCODE
                    switch (OpCode)
                    {
                        case (0x00): BRK(); break;
                        case (0x01): ORA(); break;
                        case (0x05): ORA(); break;
                        case (0x06): ASL(); break;
                        case (0x08): PHP(); break;
                        case (0x09): ORA(); break;
                        case (0x0a): ASL(); break;
                        case (0x0d): ORA(); break;
                        case (0x0e): ASL(); break;
                        case (0x10): BPL(); break;
                        case (0x11): ORA(); break;
                        case (0x15): ORA(); break;
                        case (0x16): ASL(); break;
                        case (0x18): CLC(); break;
                        case (0x19): ORA(); break;
                        case (0x1d): ORA(); break;
                        case (0x1e): ASL(); break;
                        case (0x20): JSR(); break;
                        case (0x21): AND(); break;
                        case (0x24): BIT(); break;
                        case (0x25): AND(); break;
                        case (0x26): ROL(); break;
                        case (0x28): PLP(); break;
                        case (0x29): AND(); break;
                        case (0x2a): ROL(); break;
                        case (0x2c): BIT(); break;
                        case (0x2d): AND(); break;
                        case (0x2e): ROL(); break;
                        case (0x30): BMI(); break;
                        case (0x31): AND(); break;
                        case (0x32): NOP(); break;
                        case (0x33): NOP(); break;
                        case (0x34): NOP(); break;
                        case (0x35): AND(); break;
                        case (0x36): ROL(); break;
                        case (0x38): SEC(); break;
                        case (0x39): AND(); break;
                        case (0x3d): AND(); break;
                        case (0x3e): ROL(); break;
                        case (0x40): RTI(); break;
                        case (0x41): EOR(); break;
                        case (0x45): EOR(); break;
                        case (0x46): LSR(); break;
                        case (0x48): PHA(); break;
                        case (0x49): EOR(); break;
                        case (0x4a): LSR(); break;
                        case (0x4c): JMP(); break;
                        case (0x4d): EOR(); break;
                        case (0x4e): LSR(); break;
                        case (0x50): BVC(); break;
                        case (0x51): EOR(); break;
                        case (0x55): EOR(); break;
                        case (0x56): LSR(); break;
                        case (0x58): CLI(); break;
                        case (0x59): EOR(); break;
                        case (0x5d): EOR(); break;
                        case (0x5e): LSR(); break;
                        case (0x60): RTS(); break;
                        case (0x61): ADC(); break;
                        case (0x65): ADC(); break;
                        case (0x66): ROR(); break;
                        case (0x68): PLA(); break;
                        case (0x69): ADC(); break;
                        case (0x6a): ROR(); break;
                        case (0x6c): JMP(); break;
                        case (0x6d): ADC(); break;
                        case (0x6e): ROR(); break;
                        case (0x70): BVS(); break;
                        case (0x71): ADC(); break;
                        case (0x75): ADC(); break;
                        case (0x76): ROR(); break;
                        case (0x78): SEI(); break;
                        case (0x79): ADC(); break;
                        case (0x7d): ADC(); break;
                        case (0x7e): ROR(); break;
                        case (0x81): STA(); break;
                        case (0x84): STY(); break;
                        case (0x85): STA(); break;
                        case (0x86): STX(); break;
                        case (0x88): DEY(); break;
                        case (0x8a): TXA(); break;
                        case (0x8c): STY(); break;
                        case (0x8d): STA(); break;
                        case (0x8e): STX(); break;
                        case (0x90): BCC(); break;
                        case (0x91): STA(); break;
                        case (0x94): STY(); break;
                        case (0x95): STA(); break;
                        case (0x96): STX(); break;
                        case (0x98): TYA(); break;
                        case (0x99): STA(); break;
                        case (0x9a): TXS(); break;
                        case (0x9d): STA(); break;
                        case (0xa0): LDY(); break;
                        case (0xa1): LDA(); break;
                        case (0xa2): LDX(); break;
                        case (0xa4): LDY(); break;
                        case (0xa5): LDA(); break;
                        case (0xa6): LDX(); break;
                        case (0xa8): TAY(); break;
                        case (0xa9): LDA(); break;
                        case (0xaa): TAX(); break;
                        case (0xac): LDY(); break;
                        case (0xad): LDA(); break;
                        case (0xae): LDX(); break;
                        case (0xb0): BCS(); break;
                        case (0xb1): LDA(); break;
                        case (0xb4): LDY(); break;
                        case (0xb5): LDA(); break;
                        case (0xb6): LDX(); break;
                        case (0xb8): CLV(); break;
                        case (0xb9): LDA(); break;
                        case (0xba): TSX(); break;
                        case (0xbc): LDY(); break;
                        case (0xbd): LDA(); break;
                        case (0xbe): LDX(); break;
                        case (0xc0): CPY(); break;
                        case (0xc1): CMP(); break;
                        case (0xc4): CPY(); break;
                        case (0xc5): CMP(); break;
                        case (0xc6): DEC(); break;
                        case (0xc8): INY(); break;
                        case (0xc9): CMP(); break;
                        case (0xca): DEX(); break;
                        case (0xcc): CPY(); break;
                        case (0xcd): CMP(); break;
                        case (0xce): DEC(); break;
                        case (0xd0): BNE(); break;
                        case (0xd1): CMP(); break;
                        case (0xd5): CMP(); break;
                        case (0xd6): DEC(); break;
                        case (0xd8): CLD(); break;
                        case (0xd9): CMP(); break;
                        case (0xdd): CMP(); break;
                        case (0xde): DEC(); break;
                        case (0xe0): CPX(); break;
                        case (0xe1): SBC(); break;
                        case (0xe4): CPX(); break;
                        case (0xe5): SBC(); break;
                        case (0xe6): INC(); break;
                        case (0xe8): INX(); break;
                        case (0xe9): SBC(); break;
                        case (0xec): CPX(); break;
                        case (0xed): SBC(); break;
                        case (0xee): INC(); break;
                        case (0xf0): BEQ(); break;
                        case (0xf1): SBC(); break;
                        case (0xf5): SBC(); break;
                        case (0xf6): INC(); break;
                        case (0xf8): SED(); break;
                        case (0xf9): SBC(); break;
                        case (0xfd): SBC(); break;
                        case (0xfe): INC(); break;
                        default: NOP(); break;
                    }
                    #endregion
                    _Nes.APU.Play();//If the sound is paused

                }
                else
                {
                    Thread.Sleep(100);
                    _Nes.APU.Pause();
                    Paused = true;
                }
                if (CycleCounter >= CyclesPerScanline)
                {
                    if (_Nes.PPU.DoScanline())/*NMI*/
                    {
                        Flag_B = false;
                        Push16(REG_PC);
                        PushStatus();
                        Flag_I = true;
                        REG_PC = MEM.Read16(0xFFFA);
                    }
                    if (IRQNextTime)/*IRQ*/
                    {
                        IRQ();
                        IRQNextTime = false;
                    }
                    //CycleCounter -= CyclesPerScanline;
                    CycleCounter = 0;
                }
            }
        }
        public void Reset()
        {
            REG_S = 0xFF;
            REG_A = 0;
            REG_X = 0;
            REG_Y = 0;
            Flag_N = false;
            Flag_V = false;
            Flag_B = false;
            Flag_D = false;
            Flag_I = true;
            Flag_Z = false;
            Flag_C = false;
            REG_PC = MEM.Read16(0xFFFC);
        }
        public void SoftReset()
        {
            Pause = true;
            REG_S -= 3;
            Flag_I = true;
            REG_PC = MEM.Read16(0xFFFC);
            _Nes.MEMORY.MAP.CurrentMapper.SoftReset();
            Pause = false;
        }
        /// <summary>
        /// Pause the cpu
        /// </summary>
        public void TogglePause()
        {
            _Pause = !_Pause;
            //Wait until the cpu pauses if the user resume the cpu loop
            if (_Pause)
                while (!Paused)
                { }
            //Rise the event
            if (PauseToggle != null)
                PauseToggle(this, null);
        }
        /// <summary>
        /// Initialize and set the timing of the cpu
        /// </summary>
        /// <param name="FORMAT">The tv format to set the cpu emulation for</param>
        void InitializeCPU(TVFORMAT FORMAT)
        {
            switch (FORMAT)
            {
                case TVFORMAT.NTSC:
                    CyclesPerScanline = 113;
                    break;
                case TVFORMAT.PAL:
                    CyclesPerScanline = 106;
                    break;
            }
            REG_S = 0xFF;
            REG_A = 0;
            REG_X = 0;
            REG_Y = 0;
            DEBUG.WriteLine(this, "CPU initialized ok.", DebugStatus.Cool);
        }
        /// <summary>
        /// get two bytes into a correct address
        /// </summary>
        /// <param name="c">Byte A</param>
        /// <param name="d">Byte B</param>
        /// <returns>ushort A & B togather</returns>
        ushort MakeAddress(byte c, byte d)
        {
            ushort New = (ushort)((d << 8) | c);
            return New;
        }
        public void SetTVFormat(TVFORMAT FORMAT)
        {
            if (!_Pause)
                return;
            switch (FORMAT)
            {
                case TVFORMAT.NTSC:
                    CyclesPerScanline = 113;
                    break;
                case TVFORMAT.PAL:
                    CyclesPerScanline = 106;
                    break;
            }
        }
        //Properties
        public bool ON
        { get { return _ON; } set { _ON = value; } }

        public bool Pause
        {
            get { return _Pause; }
            set
            {
                _Pause = value;
                //Wait until the cpu pauses if the user resume the cpu loop
                if (_Pause)
                    while (!Paused)
                    { }
            }
        }
        #region Addressing Modes
        byte ZeroPage(ushort A)
        { return MEM.Read(A); }
        byte ZeroPageX(ushort A)
        { return MEM.Read((ushort)(0xFF & (A + REG_X))); }
        byte ZeroPageY(ushort A)
        { return MEM.Read((ushort)(0xFF & (A + REG_Y))); }
        byte Absolute(byte A, byte B)
        { return MEM.Read(MakeAddress(A, B)); }
        byte AbsoluteX(byte A, byte B, bool CheckPage)
        {
            if (CheckPage)
            {
                if ((MakeAddress(A, B) & 0xFF00) !=
                   ((MakeAddress(A, B) + REG_X) & 0xFF00))
                {
                    CycleCounter += 1;
                };
            }
            return MEM.Read((ushort)(MakeAddress(A, B) + REG_X));
        }
        byte AbsoluteY(byte A, byte B, bool CheckPage)
        {
            if (CheckPage)
            {
                if ((MakeAddress(A, B) & 0xFF00) !=
                   ((MakeAddress(A, B) + REG_Y) & 0xFF00))
                {
                    CycleCounter += 1;
                };
            }
            return MEM.Read((ushort)(MakeAddress(A, B) + REG_Y));
        }
        byte IndirectX(byte A)
        {
            return MEM.Read((ushort)MEM.Read16((ushort)(0xff & (A + (ushort)REG_X))));
        }
        byte IndirectY(byte A, bool CheckPage)
        {
            if (CheckPage)
            {
                if ((MEM.Read16(A) & 0xFF00) !=
                    ((MEM.Read16(A) + REG_Y) & 0xFF00))
                {
                    CycleCounter += 1;
                };
            }
            return MEM.Read((ushort)(MEM.Read16(A) + (ushort)REG_Y));
        }

        byte ZeroPageWrite(ushort A, byte data)
        {
            return MEM.Write(A, data);
        }
        byte ZeroPageXWrite(ushort A, byte data)
        {
            return MEM.Write((ushort)(0xff & (A + REG_X)), data);
        }
        byte ZeroPageYWrite(ushort A, byte data)
        {
            return MEM.Write((ushort)(0xff & (A + REG_Y)), data);
        }
        byte AbsoluteWrite(byte A, byte B, byte data)
        {
            return MEM.Write(MakeAddress(A, B), data);
        }
        byte AbsoluteXWrite(byte A, byte B, byte data)
        {
            return MEM.Write((ushort)(MakeAddress(A, B) + REG_X), data);
        }
        byte AbsoluteYWrite(byte A, byte B, byte data)
        {
            return MEM.Write((ushort)(MakeAddress(A, B) + REG_Y), data);
        }
        byte IndirectXWrite(byte A, byte data)
        {
            return MEM.Write((ushort)MEM.Read16((ushort)(0xff & (A + (short)REG_X))), data);
        }
        byte IndirectYWrite(byte A, byte data)
        {
            return MEM.Write((ushort)(MEM.Read16(A) + (ushort)REG_Y), data);
        }
        #endregion
        #region OPCODES
        void ADC()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            switch (OpCode)
            {
                case (0x69): M = arg1; break;
                case (0x65): M = ZeroPage(arg1); break;
                case (0x75): M = ZeroPageX(arg1); break;
                case (0x6D): M = Absolute(arg1, arg2); break;
                case (0x7D): M = AbsoluteX(arg1, arg2, true); break;
                case (0x79): M = AbsoluteY(arg1, arg2, true); break;
                case (0x61): M = IndirectX(arg1); break;
                case (0x71): M = IndirectY(arg1, true); break;
                default: _ON = false; DEBUG.WriteLine(this, "Broken ADC", DebugStatus.Error); break;
            }
            //Execute
            uint valueholder32;
            int carry_flag = Flag_C ? 1 : 0;
            valueholder32 = (uint)(REG_A + M + carry_flag);
            //Set flags
            Flag_C = (valueholder32 > 255);
            Flag_V = (((valueholder32 ^ REG_A) & (valueholder32 ^ M)) & 0x80) != 0;
            Flag_Z = ((valueholder32 & 0xFF) == 0);
            Flag_N = ((valueholder32 & 0x80) == 0x80);
            //A + M + C -> A
            REG_A = (byte)(valueholder32 & 0xff);
            //Advance PC and tick count
            switch (OpCode)
            {
                case (0x69): CycleCounter += 2; REG_PC += 2; break;
                case (0x65): CycleCounter += 3; REG_PC += 2; break;
                case (0x75): CycleCounter += 4; REG_PC += 2; break;
                case (0x6D): CycleCounter += 4; REG_PC += 3; break;
                case (0x7D): CycleCounter += 4; REG_PC += 3; break;
                case (0x79): CycleCounter += 4; REG_PC += 3; break;
                case (0x61): CycleCounter += 6; REG_PC += 2; break;
                case (0x71): CycleCounter += 5; REG_PC += 2; break;
                default: _ON = false; break;
            }
        }
        void AND()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            switch (OpCode)
            {
                case (0x29): M = arg1; break;
                case (0x25): M = ZeroPage(arg1); break;
                case (0x35): M = ZeroPageX(arg1); break;
                case (0x2D): M = Absolute(arg1, arg2); break;
                case (0x3D): M = AbsoluteX(arg1, arg2, true); break;
                case (0x39): M = AbsoluteY(arg1, arg2, true); break;
                case (0x21): M = IndirectX(arg1); break;
                case (0x31): M = IndirectY(arg1, false); break;
                default: _ON = false; DEBUG.WriteLine(this, "Broken AND", DebugStatus.Error); break;
            }
            REG_A = (byte)(REG_A & M);
            //Flags
            Flag_Z = (REG_A == 0);
            Flag_N = ((REG_A & 0x80) == 0x80);
            //Advance PC and tick count
            switch (OpCode)
            {
                case (0x29): CycleCounter += 2; REG_PC += 2; break;
                case (0x25): CycleCounter += 3; REG_PC += 2; break;
                case (0x35): CycleCounter += 4; REG_PC += 2; break;
                case (0x2D): CycleCounter += 3; REG_PC += 3; break;
                case (0x3D): CycleCounter += 4; REG_PC += 3; break;
                case (0x39): CycleCounter += 4; REG_PC += 3; break;
                case (0x21): CycleCounter += 6; REG_PC += 2; break;
                case (0x31): CycleCounter += 5; REG_PC += 2; break;
                default: _ON = false; Console.WriteLine("Broken AND"); break;
            }
        }
        void ASL()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            switch (OpCode)
            {
                case (0x0a): M = REG_A; break;
                case (0x06): M = ZeroPage(arg1); break;
                case (0x16): M = ZeroPageX(arg1); break;
                case (0x0E): M = Absolute(arg1, arg2); break;
                case (0x1E): M = AbsoluteX(arg1, arg2, false); break;
                default: _ON = false; Console.WriteLine("Broken ASL"); break;
            }
            //Flags
            Flag_C = ((M & 0x80) == 0x80);
            M <<= 1;
            Flag_Z = ((M & 0xff) == 0x0);
            Flag_N = ((M & 0x80) == 0x80);
            //This one is a little different because we actually need
            //to do more than incrementing in the last step		
            switch (OpCode)
            {
                case (0x0a): REG_A = M;
                    CycleCounter += 2; REG_PC += 1; break;
                case (0x06): ZeroPageWrite(arg1, M);
                    CycleCounter += 5; REG_PC += 2; break;
                case (0x16): ZeroPageXWrite(arg1, M);
                    CycleCounter += 6; REG_PC += 2; break;
                case (0x0E): AbsoluteWrite(arg1, arg2, M);
                    CycleCounter += 6; REG_PC += 3; break;
                case (0x1E): AbsoluteXWrite(arg1, arg2, M);
                    CycleCounter += 7; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken ASL"); break;
            }
        }
        void BCC()
        {
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            if (!Flag_C)
            {
                REG_PC += 2;
                if ((REG_PC & 0xFF00) != ((REG_PC + (sbyte)arg1 + 2) & 0xFF00))
                {
                    CycleCounter += 1;
                }
                REG_PC = (ushort)(REG_PC + (sbyte)arg1);
                CycleCounter += 1;
            }
            else
            {
                REG_PC += 2;
            }
            CycleCounter += 2;
        }
        void BCS()
        {
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            if (Flag_C)
            {
                REG_PC += 2;
                if ((REG_PC & 0xFF00) != ((REG_PC + (sbyte)arg1 + 2) & 0xFF00))
                {
                    CycleCounter += 1;
                }
                REG_PC = (ushort)(REG_PC + (sbyte)arg1);
                CycleCounter += 1;
            }
            else
            {
                REG_PC += 2;
            }
            CycleCounter += 2;
        }
        void BEQ()
        {
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            if (Flag_Z)
            {
                REG_PC += 2;
                if ((REG_PC & 0xFF00) != ((REG_PC + (sbyte)arg1 + 2) & 0xFF00))
                {
                    CycleCounter += 1;
                }
                REG_PC = (ushort)(REG_PC + (sbyte)arg1);
                CycleCounter += 1;
            }
            else
            {
                REG_PC += 2;
            }
            CycleCounter += 2;
        }
        void BIT()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder

            switch (OpCode)
            {
                case (0x24): M = ZeroPage(arg1); break;
                case (0x2c): M = Absolute(arg1, arg2); break;
                default: _ON = false; Console.WriteLine("Broken BIT"); break;
            }
            Flag_Z = ((REG_A & M) == 0x0);
            Flag_N = ((M & 0x80) == 0x80);
            Flag_V = ((M & 0x40) == 0x40);
            switch (OpCode)
            {
                case (0x24): CycleCounter += 3; REG_PC += 2; break;
                case (0x2c): CycleCounter += 4; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken BIT"); break;
            }
        }
        void BMI()
        {
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            if (Flag_N)
            {
                REG_PC += 2;
                if ((REG_PC & 0xFF00) != ((REG_PC + (sbyte)arg1 + 2) & 0xFF00))
                {
                    CycleCounter += 1;
                }
                REG_PC = (ushort)(REG_PC + (sbyte)arg1);
                CycleCounter += 1;
            }
            else
            {
                REG_PC += 2;
            }
            CycleCounter += 2;
        }
        void BNE()
        {
            byte arg1;
            if (!Flag_Z)
            {
                arg1 = MEM.Read((ushort)(REG_PC + 1));
                REG_PC += 2;
                if ((REG_PC & 0xFF00) != ((REG_PC + (sbyte)arg1 + 2) & 0xFF00))
                {
                    CycleCounter += 1;
                }
                REG_PC = (ushort)(REG_PC + (sbyte)arg1);
                CycleCounter += 1;
            }
            else
            {
                REG_PC += 2;
            }
            CycleCounter += 2;
        }
        void BPL()
        {
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            if (!Flag_N)
            {
                REG_PC += 2;
                if ((REG_PC & 0xFF00) != ((REG_PC + (sbyte)arg1 + 2) & 0xFF00))
                {
                    CycleCounter += 1;
                }
                REG_PC = (ushort)(REG_PC + (sbyte)arg1);
                CycleCounter += 1;
            }
            else
            {
                REG_PC += 2;
            }
            CycleCounter += 2;
        }
        void BRK()
        {
            REG_PC++;
            Push16((ushort)(REG_PC + 1));
            Flag_B = true;
            PushStatus();
            Flag_I = true;
            REG_PC = MEM.Read16(0xFFFE);
            CycleCounter += 7;
        }
        void BVC()
        {
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            if (!Flag_V)
            {
                REG_PC += 2;
                if ((REG_PC & 0xFF00) != ((REG_PC + (sbyte)arg1 + 2) & 0xFF00))
                {
                    CycleCounter += 1;
                }
                REG_PC = (ushort)(REG_PC + (sbyte)arg1);
                CycleCounter += 1;
            }
            else
            {
                REG_PC += 2;
            }
            CycleCounter += 2;
        }
        void BVS()
        {
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            if (Flag_V)
            {
                REG_PC += 2;
                if ((REG_PC & 0xFF00) != ((REG_PC + (sbyte)arg1 + 2) & 0xFF00))
                {
                    CycleCounter += 1;
                }
                REG_PC = (ushort)(REG_PC + (sbyte)arg1);
                CycleCounter += 1;
            }
            else
            {
                REG_PC += 2;
            }
            CycleCounter += 2;
        }
        void CLC()
        {
            Flag_C = false;
            REG_PC += 1;
            CycleCounter += 2;
        }
        void CLD()
        {
            Flag_D = false;
            REG_PC += 1;
            CycleCounter += 2;
        }
        void CLI()
        {
            Flag_I = false;
            REG_PC += 1;
            CycleCounter += 2;
        }
        void CLV()
        {
            Flag_V = false;
            REG_PC += 1;
            CycleCounter += 2;
        }
        void CMP()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder

            switch (OpCode)
            {
                case (0xC9): M = arg1; break;
                case (0xC5): M = ZeroPage(arg1); break;
                case (0xD5): M = ZeroPageX(arg1); break;
                case (0xCD): M = Absolute(arg1, arg2); break;
                case (0xDD): M = AbsoluteX(arg1, arg2, true); break;
                case (0xD9): M = AbsoluteY(arg1, arg2, true); break;
                case (0xC1): M = IndirectX(arg1); break;
                case (0xD1): M = IndirectY(arg1, true); break;
                default: _ON = false; Console.WriteLine("Broken CMP"); break;
            }
            //Flags
            Flag_C = (REG_A >= M);
            Flag_Z = (REG_A == M);
            M = (byte)(REG_A - M);
            Flag_N = ((M & 0x80) == 0x80);
            switch (OpCode)
            {
                case (0xC9): CycleCounter += 2; REG_PC += 2; break;
                case (0xC5): CycleCounter += 3; REG_PC += 2; break;
                case (0xD5): CycleCounter += 4; REG_PC += 2; break;
                case (0xCD): CycleCounter += 4; REG_PC += 3; break;
                case (0xDD): CycleCounter += 4; REG_PC += 3; break;
                case (0xD9): CycleCounter += 4; REG_PC += 3; break;
                case (0xC1): CycleCounter += 6; REG_PC += 2; break;
                case (0xD1): CycleCounter += 5; REG_PC += 2; break;
                default: _ON = false; Console.WriteLine("Broken CMP"); break;
            }
        }
        void CPX()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder

            switch (OpCode)
            {
                case (0xE0): M = arg1; break;
                case (0xE4): M = ZeroPage(arg1); break;
                case (0xEC): M = Absolute(arg1, arg2); break;
                default: _ON = false; Console.WriteLine("Broken CPX"); break;
            }
            Flag_C = (REG_X >= M);

            //Flag_Z = (M == 0);
            Flag_Z = (REG_X == M);
            M = (byte)(REG_X - M);
            Flag_N = ((M & 0x80) == 0x80);
            switch (OpCode)
            {
                case (0xE0): CycleCounter += 2; REG_PC += 2; break;
                case (0xE4): CycleCounter += 3; REG_PC += 2; break;
                case (0xEC): CycleCounter += 4; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken CPX"); break;
            }
        }
        void CPY()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            switch (OpCode)
            {
                case (0xC0): M = arg1; break;
                case (0xC4): M = ZeroPage(arg1); break;
                case (0xCC): M = Absolute(arg1, arg2); break;
                default: _ON = false; Console.WriteLine("Broken CPY"); break;
            }

            Flag_C = (REG_Y >= M);
            Flag_Z = (REG_Y == M);
            M = (byte)(REG_Y - M);
            Flag_N = ((M & 0x80) == 0x80);
            switch (OpCode)
            {
                case (0xC0): CycleCounter += 2; REG_PC += 2; break;
                case (0xC4): CycleCounter += 3; REG_PC += 2; break;
                case (0xCC): CycleCounter += 4; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken CPY"); break;
            }
        }
        void DEC()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            switch (OpCode)
            {
                case (0xC6): M = ZeroPage(arg1); break;
                case (0xD6): M = ZeroPageX(arg1); break;
                case (0xCE): M = Absolute(arg1, arg2); break;
                case (0xDE): M = AbsoluteX(arg1, arg2, false); break;
                default: _ON = false; Console.WriteLine("Broken DEC"); break;
            }
            M--;
            Flag_Z = (M == 0x0);
            Flag_N = ((M & 0x80) == 0x80);
            switch (OpCode)
            {
                case (0xC6): ZeroPageWrite(arg1, M);
                    CycleCounter += 5; REG_PC += 2; break;
                case (0xD6): ZeroPageXWrite(arg1, M);
                    CycleCounter += 6; REG_PC += 2; break;
                case (0xCE): AbsoluteWrite(arg1, arg2, M);
                    CycleCounter += 6; REG_PC += 3; break;
                case (0xDE): AbsoluteXWrite(arg1, arg2, M);
                    CycleCounter += 7; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken DEC"); break;
            }
        }
        void DEX()
        {
            REG_X--;
            Flag_Z = (REG_X == 0x0);
            Flag_N = ((REG_X & 0x80) == 0x80);
            REG_PC++;
            CycleCounter += 2;
        }
        void DEY()
        {
            REG_Y--;
            Flag_Z = (REG_Y == 0x0);
            Flag_N = ((REG_Y & 0x80) == 0x80);
            REG_PC++;
            CycleCounter += 2;
        }
        void EOR()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            switch (OpCode)
            {
                case (0x49): M = arg1; break;
                case (0x45): M = ZeroPage(arg1); break;
                case (0x55): M = ZeroPageX(arg1); break;
                case (0x4D): M = Absolute(arg1, arg2); break;
                case (0x5D): M = AbsoluteX(arg1, arg2, true); break;
                case (0x59): M = AbsoluteY(arg1, arg2, true); break;
                case (0x41): M = IndirectX(arg1); break;
                case (0x51): M = IndirectY(arg1, true); break;
                default: _ON = false; Console.WriteLine("Broken EOR"); break;
            }
            REG_A = (byte)(REG_A ^ M);
            Flag_Z = (REG_A == 0);
            Flag_N = ((REG_A & 0x80) == 0x80);
            switch (OpCode)
            {
                case (0x49): CycleCounter += 2; REG_PC += 2; break;
                case (0x45): CycleCounter += 3; REG_PC += 2; break;
                case (0x55): CycleCounter += 4; REG_PC += 2; break;
                case (0x4D): CycleCounter += 3; REG_PC += 3; break;
                case (0x5D): CycleCounter += 4; REG_PC += 3; break;
                case (0x59): CycleCounter += 4; REG_PC += 3; break;
                case (0x41): CycleCounter += 6; REG_PC += 2; break;
                case (0x51): CycleCounter += 5; REG_PC += 2; break;
                default: _ON = false; Console.WriteLine("Broken EOR"); break;
            }
        }
        void INC()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            switch (OpCode)
            {
                case (0xE6): M = ZeroPage(arg1); break;
                case (0xF6): M = ZeroPageX(arg1); break;
                case (0xEE): M = Absolute(arg1, arg2); break;
                case (0xFE): M = AbsoluteX(arg1, arg2, false); break;
                default: _ON = false; Console.WriteLine("Broken INC"); break;
            }
            M++;
            Flag_Z = ((M & 0xff) == 0x0);
            Flag_N = ((M & 0x80) == 0x80);
            switch (OpCode)
            {
                case (0xE6): ZeroPageWrite(arg1, M);
                    CycleCounter += 5; REG_PC += 2; break;
                case (0xF6): ZeroPageXWrite(arg1, M);
                    CycleCounter += 6; REG_PC += 2; break;
                case (0xEE): AbsoluteWrite(arg1, arg2, M);
                    CycleCounter += 6; REG_PC += 3; break;
                case (0xFE): AbsoluteXWrite(arg1, arg2, M);
                    CycleCounter += 7; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken INC"); break;
            }
        }
        void INX()
        {
            REG_X++;
            Flag_Z = ((REG_X & 0xff) == 0x0);
            Flag_N = ((REG_X & 0x80) == 0x80);
            REG_PC++;
            CycleCounter += 2;
        }
        void INY()
        {
            REG_Y++;
            Flag_Z = ((REG_Y & 0xff) == 0x0);
            Flag_N = ((REG_Y & 0x80) == 0x80);
            REG_PC++;
            CycleCounter += 2;
        }
        void IRQ()
        {
            if (!Flag_I)
            {
                Flag_B = false;
                Push16(REG_PC);
                PushStatus();
                Flag_I = true;
                REG_PC = MEM.Read16(0xFFFE);
            }
        }
        void JMP()
        {
            ushort myAddress = MEM.Read16((ushort)(REG_PC + 1));
            switch (OpCode)
            {
                case (0x4c):
                    REG_PC = myAddress;
                    CycleCounter += 3; break;
                case (0x6c):
                    if ((myAddress & 0x00FF) == 0x00FF)
                    {
                        REG_PC = MEM.Read(myAddress);
                        myAddress &= 0xFF00;
                        REG_PC |= (ushort)((MEM.Read(myAddress)) << 8);
                    }
                    else
                        REG_PC = MEM.Read16(myAddress);
                    CycleCounter += 5; break;
                default: _ON = false; Console.WriteLine("Broken JMP"); break;
            }
        }
        void JSR()
        {
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            Push16((ushort)(REG_PC + 2));
            REG_PC = MakeAddress(arg1, arg2);
            CycleCounter += 6;
        }
        void LDA()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2;
            switch (OpCode)
            {
                case (0xA9): REG_A = arg1;
                    CycleCounter += 2; REG_PC += 2; break;
                case (0xA5): REG_A = ZeroPage(arg1);
                    CycleCounter += 3; REG_PC += 2; break;
                case (0xB5): REG_A = ZeroPageX(arg1);
                    CycleCounter += 4; REG_PC += 2; break;
                case (0xAD): arg2 = MEM.Read((ushort)(REG_PC + 2));
                    REG_A = Absolute(arg1, arg2);
                    CycleCounter += 4; REG_PC += 3; break;
                case (0xBD): arg2 = MEM.Read((ushort)(REG_PC + 2));
                    REG_A = AbsoluteX(arg1, arg2, true);     //CHECK FOR PAGE BOUNDARIES
                    CycleCounter += 4; REG_PC += 3; break;
                case (0xB9): arg2 = MEM.Read((ushort)(REG_PC + 2));
                    REG_A = AbsoluteY(arg1, arg2, true);     //CHECK FOR PAGE BOUNDARIES
                    CycleCounter += 4; REG_PC += 3; break;
                case (0xA1): REG_A = IndirectX(arg1);
                    CycleCounter += 6; REG_PC += 2; break;
                case (0xB1): REG_A = IndirectY(arg1, true);     //CHECK FOR PAGE BOUNDARIES
                    CycleCounter += 5; REG_PC += 2; break;
                default: _ON = false; Console.WriteLine("Broken LDA"); break;
            }
            Flag_Z = (REG_A == 0);
            Flag_N = ((REG_A & 0x80) == 0x80);
        }
        void LDX()
        {
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            switch (OpCode)
            {
                case (0xA2): REG_X = arg1;
                    CycleCounter += 2; REG_PC += 2; break;
                case (0xA6): REG_X = ZeroPage(arg1);
                    CycleCounter += 3; REG_PC += 2; break;
                case (0xB6): REG_X = ZeroPageY(arg1);
                    CycleCounter += 4; REG_PC += 2; break;
                case (0xAE): REG_X = Absolute(arg1, arg2);
                    CycleCounter += 4; REG_PC += 3; break;
                case (0xBE): REG_X = AbsoluteY(arg1, arg2, true);
                    CycleCounter += 4; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken LDX"); break;
            }
            Flag_Z = (REG_X == 0);
            Flag_N = ((REG_X & 0x80) == 0x80);
        }
        void LDY()
        {
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            switch (OpCode)
            {
                case (0xA0): REG_Y = arg1;
                    CycleCounter += 2; REG_PC += 2; break;
                case (0xA4): REG_Y = ZeroPage(arg1);
                    CycleCounter += 3; REG_PC += 2; break;
                case (0xB4): REG_Y = ZeroPageX(arg1);
                    CycleCounter += 4; REG_PC += 2; break;
                case (0xAC): REG_Y = Absolute(arg1, arg2);
                    CycleCounter += 4; REG_PC += 3; break;
                case (0xBC): REG_Y = AbsoluteX(arg1, arg2, true);
                    CycleCounter += 4; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken LDY"); break;
            }
            Flag_Z = (REG_Y == 0);
            Flag_N = ((REG_Y & 0x80) == 0x80);
        }
        void LSR()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            switch (OpCode)
            {
                case (0x4a): M = REG_A; break;
                case (0x46): M = ZeroPage(arg1); break;
                case (0x56): M = ZeroPageX(arg1); break;
                case (0x4E): M = Absolute(arg1, arg2); break;
                case (0x5E): M = AbsoluteX(arg1, arg2, false); break;
                default: _ON = false; Console.WriteLine("Broken LSR"); break;
            }
            Flag_C = ((M & 0x1) == 0x1);
            M = (byte)(M >> 1);
            Flag_Z = ((M & 0xff) == 0x0);
            Flag_N = ((M & 0x80) == 0x80);
            switch (OpCode)
            {
                case (0x4a): REG_A = M;
                    CycleCounter += 2; REG_PC += 1; break;
                case (0x46): ZeroPageWrite(arg1, M);
                    CycleCounter += 5; REG_PC += 2; break;
                case (0x56): ZeroPageXWrite(arg1, M);
                    CycleCounter += 6; REG_PC += 2; break;
                case (0x4E): AbsoluteWrite(arg1, arg2, M);
                    CycleCounter += 6; REG_PC += 3; break;
                case (0x5E): AbsoluteXWrite(arg1, arg2, M);
                    CycleCounter += 7; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken LSR"); break;
            }
        }
        void NOP()
        {
            REG_PC += 1;
            CycleCounter += 2;
        }
        void ORA()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            switch (OpCode)
            {
                case (0x09): M = arg1; break;
                case (0x05): M = ZeroPage(arg1); break;
                case (0x15): M = ZeroPageX(arg1); break;
                case (0x0D): M = Absolute(arg1, arg2); break;
                case (0x1D): M = AbsoluteX(arg1, arg2, true); break;
                case (0x19): M = AbsoluteY(arg1, arg2, true); break;
                case (0x01): M = IndirectX(arg1); break;
                case (0x11): M = IndirectY(arg1, false); break;
                default: _ON = false; Console.WriteLine("Broken ORA"); break;
            }

            REG_A = (byte)(REG_A | M);
            Flag_Z = ((REG_A & 0xff) == 0);
            Flag_N = ((REG_A & 0x80) == 0x80);
            switch (OpCode)
            {
                case (0x09): CycleCounter += 2; REG_PC += 2; break;
                case (0x05): CycleCounter += 3; REG_PC += 2; break;
                case (0x15): CycleCounter += 4; REG_PC += 2; break;
                case (0x0D): CycleCounter += 4; REG_PC += 3; break;
                case (0x1D): CycleCounter += 4; REG_PC += 3; break;
                case (0x19): CycleCounter += 4; REG_PC += 3; break;
                case (0x01): CycleCounter += 6; REG_PC += 2; break;
                case (0x11): CycleCounter += 5; REG_PC += 2; break;
                default: _ON = false; Console.WriteLine("Broken ORA"); break;
            }
        }
        void PHA()
        {
            Push8(REG_A);
            REG_PC += 1;
            CycleCounter += 3;
        }
        void PHP()
        {
            Flag_B = true;
            PushStatus();
            REG_PC += 1;
            CycleCounter += 3;
        }
        void PLA()
        {
            REG_A = Pull8();
            Flag_Z = (REG_A == 0);
            Flag_N = ((REG_A & 0x80) == 0x80);
            REG_PC += 1;
            CycleCounter += 4;
        }
        void PLP()
        {
            PullStatus();
            REG_PC += 1;
            CycleCounter += 4;
        }
        void ROL()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            byte bitholder = 0;

            switch (OpCode)
            {
                case (0x2a): M = REG_A; break;
                case (0x26): M = ZeroPage(arg1); break;
                case (0x36): M = ZeroPageX(arg1); break;
                case (0x2E): M = Absolute(arg1, arg2); break;
                case (0x3E): M = AbsoluteX(arg1, arg2, false); break;
                default: _ON = false; Console.WriteLine("Broken ROL"); break;
            }
            if ((M & 0x80) == 0x80)
                bitholder = 1;
            else
                bitholder = 0;
            byte carry_flag = (byte)(Flag_C ? 1 : 0);
            M = (byte)(M << 1);
            M = (byte)(M | carry_flag);
            carry_flag = bitholder;
            Flag_C = (bitholder == 1);
            Flag_Z = ((M & 0xff) == 0x0);
            Flag_N = ((M & 0x80) == 0x80);
            switch (OpCode)
            {
                case (0x2a): REG_A = M;
                    CycleCounter += 2; REG_PC += 1; break;
                case (0x26): ZeroPageWrite(arg1, M);
                    CycleCounter += 5; REG_PC += 2; break;
                case (0x36): ZeroPageXWrite(arg1, M);
                    CycleCounter += 6; REG_PC += 2; break;
                case (0x2E): AbsoluteWrite(arg1, arg2, M);
                    CycleCounter += 6; REG_PC += 3; break;
                case (0x3E): AbsoluteXWrite(arg1, arg2, M);
                    CycleCounter += 7; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken ROL"); break;
            }
        }
        void ROR()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            byte bitholder = 0;
            switch (OpCode)
            {
                case (0x6a): M = REG_A; break;
                case (0x66): M = ZeroPage(arg1); break;
                case (0x76): M = ZeroPageX(arg1); break;
                case (0x6E): M = Absolute(arg1, arg2); break;
                case (0x7E): M = AbsoluteX(arg1, arg2, false); break;
                default: _ON = false; Console.WriteLine("Broken ROR"); break;
            }
            byte carry_flag = (byte)(Flag_C ? 1 : 0);
            if ((M & 0x1) == 0x1)
                bitholder = 1;
            else
                bitholder = 0;
            M = (byte)(M >> 1);

            if (carry_flag == 1)
                M = (byte)(M | 0x80);
            Flag_C = (bitholder == 1);
            Flag_Z = ((M & 0xff) == 0x0);
            Flag_N = ((M & 0x80) == 0x80);
            switch (OpCode)
            {
                case (0x6a): REG_A = M;
                    CycleCounter += 2; REG_PC += 1; break;
                case (0x66): ZeroPageWrite(arg1, M);
                    CycleCounter += 5; REG_PC += 2; break;
                case (0x76): ZeroPageXWrite(arg1, M);
                    CycleCounter += 6; REG_PC += 2; break;
                case (0x6E): AbsoluteWrite(arg1, arg2, M);
                    CycleCounter += 6; REG_PC += 3; break;
                case (0x7E): AbsoluteXWrite(arg1, arg2, M);
                    CycleCounter += 7; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken ROR"); break;
            }
        }
        void RTI()
        {
            PullStatus();
            REG_PC = Pull16();
            CycleCounter += 6;
        }
        void RTS()
        {
            REG_PC = Pull16();
            CycleCounter += 6;
            REG_PC += 1;
        }
        void SBC()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            byte M = 0xFF;//The value holder
            //Decode
            switch (OpCode)
            {
                case (0xE9): M = arg1; break;
                case (0xE5): M = ZeroPage(arg1); break;
                case (0xF5): M = ZeroPageX(arg1); break;
                case (0xED): M = Absolute(arg1, arg2); break;
                case (0xFD): M = AbsoluteX(arg1, arg2, true); break;
                case (0xF9): M = AbsoluteY(arg1, arg2, true); break;
                case (0xE1): M = IndirectX(arg1); break;
                case (0xF1): M = IndirectY(arg1, false); break;
                default: _ON = false; Console.WriteLine("Broken SBC"); break;
            }
            //Execute
            uint valueholder32;
            valueholder32 = (uint)(REG_A - M);
            if (!Flag_C)
                valueholder32 = valueholder32 - 1;

            if (valueholder32 > 255)
                Flag_C = false;
            else
                Flag_C = true;

            Flag_Z = ((valueholder32 & 0xff) == 0);
            Flag_V = ((REG_A ^ M) & (REG_A ^ valueholder32) & 0x80) != 0;
            Flag_N = ((valueholder32 & 0x80) == 0x80);
            REG_A = (byte)(valueholder32 & 0xff);
            switch (OpCode)
            {
                case (0xE9): CycleCounter += 2; REG_PC += 2; break;
                case (0xE5): CycleCounter += 3; REG_PC += 2; break;
                case (0xF5): CycleCounter += 4; REG_PC += 2; break;
                case (0xED): CycleCounter += 4; REG_PC += 3; break;
                case (0xFD): CycleCounter += 4; REG_PC += 3; break;
                case (0xF9): CycleCounter += 4; REG_PC += 3; break;
                case (0xE1): CycleCounter += 6; REG_PC += 2; break;
                case (0xF1): CycleCounter += 5; REG_PC += 2; break;
                default: _ON = false; Console.WriteLine("Broken SBC"); break;
            }
        }
        void SEC()
        {
            Flag_C = true;
            CycleCounter += 2;
            REG_PC += 1;
        }
        void SED()
        {
            Flag_D = true;
            CycleCounter += 2;
            REG_PC += 1;
        }
        void SEI()
        {
            Flag_I = true;
            CycleCounter += 2;
            REG_PC += 1;
        }
        void STA()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            //Decode
            switch (OpCode)
            {
                case (0x85): ZeroPageWrite(arg1, REG_A);
                    CycleCounter += 3; REG_PC += 2; break;
                case (0x95): ZeroPageXWrite(arg1, REG_A);
                    CycleCounter += 4; REG_PC += 2; break;
                case (0x8D): AbsoluteWrite(arg1, arg2, REG_A);
                    CycleCounter += 4; REG_PC += 3; break;
                case (0x9D): AbsoluteXWrite(arg1, arg2, REG_A);
                    CycleCounter += 5; REG_PC += 3; break;
                case (0x99): AbsoluteYWrite(arg1, arg2, REG_A);
                    CycleCounter += 5; REG_PC += 3; break;
                case (0x81): IndirectXWrite(arg1, REG_A);
                    CycleCounter += 6; REG_PC += 2; break;
                case (0x91): IndirectYWrite(arg1, REG_A);
                    CycleCounter += 6; REG_PC += 2; break;
                default: _ON = false; Console.WriteLine("Broken STA"); break;
            }
        }
        void STX()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            //Decode
            switch (OpCode)
            {
                case (0x86): ZeroPageWrite(arg1, REG_X);
                    CycleCounter += 3; REG_PC += 2; break;
                case (0x96): ZeroPageYWrite(arg1, REG_X);
                    CycleCounter += 4; REG_PC += 2; break;
                case (0x8E): AbsoluteWrite(arg1, arg2, REG_X);
                    CycleCounter += 4; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken STX"); break;
            }
        }
        void STY()
        {
            // We may not use both, but it's easier to grab them now
            byte arg1 = MEM.Read((ushort)(REG_PC + 1));
            byte arg2 = MEM.Read((ushort)(REG_PC + 2));
            //Decode
            switch (OpCode)
            {
                case (0x84): ZeroPageWrite(arg1, REG_Y);
                    CycleCounter += 3; REG_PC += 2; break;
                case (0x94): ZeroPageXWrite(arg1, REG_Y);
                    CycleCounter += 4; REG_PC += 2; break;
                case (0x8C): AbsoluteWrite(arg1, arg2, REG_Y);
                    CycleCounter += 4; REG_PC += 3; break;
                default: _ON = false; Console.WriteLine("Broken STY"); break;
            }
        }
        void TAX()
        {
            REG_X = REG_A;
            Flag_Z = (REG_X == 0);
            Flag_N = ((REG_X & 0x80) == 0x80);
            REG_PC += 1;
            CycleCounter += 2;
        }
        void TAY()
        {
            REG_Y = REG_A;
            Flag_Z = (REG_Y == 0);
            Flag_N = ((REG_Y & 0x80) == 0x80);
            REG_PC += 1;
            CycleCounter += 2;
        }
        void TSX()
        {
            REG_X = REG_S;
            Flag_Z = (REG_X == 0);
            Flag_N = ((REG_X & 0x80) == 0x80);
            REG_PC += 1;
            CycleCounter += 2;
        }
        void TXA()
        {
            REG_A = REG_X;
            Flag_Z = (REG_A == 0);
            Flag_N = ((REG_A & 0x80) == 0x80);
            REG_PC += 1;
            CycleCounter += 2;
        }
        void TXS()
        {
            REG_S = REG_X;
            REG_PC += 1;
            CycleCounter += 2;
        }
        void TYA()
        {
            REG_A = REG_Y;
            Flag_Z = (REG_A == 0);
            Flag_N = ((REG_A & 0x80) == 0x80);
            REG_PC += 1;
            CycleCounter += 2;
        }
        #endregion
        #region Operations (pull, Push ...)
        void Push8(byte data)
        {
            MEM.Write((ushort)(0x100 + REG_S--), data);
        }
        public void Push16(ushort data)
        {
            Push8((byte)((data & 0xFF00) >> 8));
            Push8((byte)(data & 0xff));
        }
        public void PushStatus()
        {
            byte statusdata = 0;

            if (Flag_N)
                statusdata = (byte)(statusdata | 0x80);

            if (Flag_V)
                statusdata = (byte)(statusdata | 0x40);

            statusdata = (byte)(statusdata | 0x20);

            if (Flag_B)
                statusdata = (byte)(statusdata | 0x10);

            if (Flag_D)
                statusdata = (byte)(statusdata | 0x8);

            if (Flag_I)
                statusdata = (byte)(statusdata | 0x4);

            if (Flag_Z)
                statusdata = (byte)(statusdata | 0x2);

            if (Flag_C)
                statusdata = (byte)(statusdata | 0x1);
            Push8(statusdata);
        }
        public byte StatusRegister()
        {
            byte statusdata = 0;

            if (Flag_N)
                statusdata = (byte)(statusdata | 0x80);

            if (Flag_V)
                statusdata = (byte)(statusdata | 0x40);

            statusdata = (byte)(statusdata | 0x20);

            if (Flag_B)
                statusdata = (byte)(statusdata | 0x10);

            if (Flag_D)
                statusdata = (byte)(statusdata | 0x8);

            if (Flag_I)
                statusdata = (byte)(statusdata | 0x4);

            if (Flag_Z)
                statusdata = (byte)(statusdata | 0x2);

            if (Flag_C)
                statusdata = (byte)(statusdata | 0x1);

            return statusdata;
        }
        byte Pull8()
        {
            REG_S++;
            return MEM.Read((ushort)(0x100 + REG_S));
        }
        ushort Pull16()
        {
            byte data1, data2;
            ushort fulldata;

            data1 = Pull8();
            data2 = Pull8();

            //We use MakeAddress because it's easier
            fulldata = MakeAddress(data1, data2);

            return fulldata;
        }
        void PullStatus()
        {
            byte statusdata = Pull8();
            Flag_N = ((statusdata & 0x80) == 0x80);
            Flag_V = ((statusdata & 0x40) == 0x40);
            Flag_B = ((statusdata & 0x10) == 0x10);
            Flag_D = ((statusdata & 0x8) == 0x8);
            Flag_I = ((statusdata & 0x4) == 0x4);
            Flag_Z = ((statusdata & 0x2) == 0x2);
            Flag_C = ((statusdata & 0x1) == 0x1);
        }
        #endregion
        /// <summary>
        /// Event rised when the cpu is paused or rusumed
        /// </summary>
        public event EventHandler<EventArgs> PauseToggle;
    }
}