﻿/*
This file is part of My Nes
A Nintendo Entertainment System Emulator.

 Copyright © 2009 - 2010 Ala Hadid (AHD)

My Nes is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

My Nes is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyNes.Nes
{
    public class MAP
    {
        public MAP(Memory Mem, NesEngine Nes)
        {
            _Cartridge = new Cart(Mem);
            _Nes = Nes;
        }
        Cart _Cartridge;
        IMapper _CurrentMapper;
        Mapper09 Map9;
        Mapper10 Map10;
        NesEngine _Nes;
        bool _IsSRAMReadOnly = false;
        public uint[] current_prg_rom_page = new uint[8];
        public uint[] current_chr_rom_page = new uint[8];

        public void WritePRG(ushort Address, byte Value)
        { _CurrentMapper.Write(Address, Value); }
        public void WriteCHR(ushort Address, byte Value)
        {
            if (_Cartridge.IsVRAM)
            {
                if (Address < 0x400)
                {
                    _Cartridge.CHR[current_chr_rom_page[0]][Address] = Value;
                }
                else if (Address < 0x800)
                {
                    _Cartridge.CHR[current_chr_rom_page[1]][Address - 0x400] = Value;
                }
                else if (Address < 0xC00)
                {
                    _Cartridge.CHR[current_chr_rom_page[2]][Address - 0x800] = Value;
                }
                else if (Address < 0x1000)
                {
                    _Cartridge.CHR[current_chr_rom_page[3]][Address - 0xC00] = Value;
                }
                else if (Address < 0x1400)
                {
                    _Cartridge.CHR[current_chr_rom_page[4]][Address - 0x1000] = Value;
                }
                else if (Address < 0x1800)
                {
                    _Cartridge.CHR[current_chr_rom_page[5]][Address - 0x1400] = Value;
                }
                else if (Address < 0x1C00)
                {
                    _Cartridge.CHR[current_chr_rom_page[6]][Address - 0x1800] = Value;
                }
                else
                {
                    _Cartridge.CHR[current_chr_rom_page[7]][Address - 0x1C00] = Value;
                }
            }
        }
        public byte ReadPRG(ushort Address)
        {
            byte returnvalue = 0xff;
            if (Address < 0x9000)
            {
                returnvalue = _Cartridge.PRG[current_prg_rom_page[0]][Address - 0x8000];
            }
            else if (Address < 0xA000)
            {
                returnvalue = _Cartridge.PRG[current_prg_rom_page[1]][Address - 0x9000];
            }
            else if (Address < 0xB000)
            {
                returnvalue = _Cartridge.PRG[current_prg_rom_page[2]][Address - 0xA000];
            }
            else if (Address < 0xC000)
            {
                returnvalue = _Cartridge.PRG[current_prg_rom_page[3]][Address - 0xB000];
            }
            else if (Address < 0xD000)
            {
                returnvalue = _Cartridge.PRG[current_prg_rom_page[4]][Address - 0xC000];
            }
            else if (Address < 0xE000)
            {
                returnvalue = _Cartridge.PRG[current_prg_rom_page[5]][Address - 0xD000];
            }
            else if (Address < 0xF000)
            {
                returnvalue = _Cartridge.PRG[current_prg_rom_page[6]][Address - 0xE000];
            }
            else 
            {
                returnvalue = _Cartridge.PRG[current_prg_rom_page[7]][Address - 0xF000];
            }
            return returnvalue;
        }
        public byte ReadCHR(ushort Address)
        {
            byte returnvalue = 0xff;

            if (Address < 0x400)
            {
                returnvalue = _Cartridge.CHR[current_chr_rom_page[0]][Address];
            }
            else if (Address < 0x800)
            {
                returnvalue = _Cartridge.CHR[current_chr_rom_page[1]][Address - 0x400];
            }
            else if (Address < 0xC00)
            {
                returnvalue = _Cartridge.CHR[current_chr_rom_page[2]][Address - 0x800];
            }
            else if (Address < 0x1000)
            {
                if (_Cartridge.MapperNo == 9)
                {
                    if ((Address >= 0xfd0) && (Address <= 0xfdf))
                    {
                        Map9.latch1 = 0xfd;
                        Switch4kChrRom(Map9.latch1data1, 1);
                    }
                    else if ((Address >= 0xfe0) && (Address <= 0xfef))
                    {
                        Map9.latch1 = 0xfe;
                        Switch4kChrRom(Map9.latch1data2, 1);
                    }
                }
                else if (_Cartridge.MapperNo == 10)
                {
                    if ((Address >= 0xfd0) && (Address <= 0xfdf))
                    {
                        Map10.latch1 = 0xfd;
                        Switch4kChrRom(Map10.latch1data1, 0);
                    }
                    else if ((Address >= 0xfe0) && (Address <= 0xfef))
                    {
                        Map10.latch1 = 0xfe;
                        Switch4kChrRom(Map10.latch1data2, 0);
                    }
                }
                returnvalue = _Cartridge.CHR[current_chr_rom_page[3]][Address - 0xC00];
            }
            else if (Address < 0x1400)
            {
                returnvalue = _Cartridge.CHR[current_chr_rom_page[4]][Address - 0x1000];
            }
            else if (Address < 0x1800)
            {
                returnvalue = _Cartridge.CHR[current_chr_rom_page[5]][Address - 0x1400];
            }
            else if (Address < 0x1C00)
            {
                returnvalue = _Cartridge.CHR[current_chr_rom_page[6]][Address - 0x1800];
            }
            else
            {
                if (_Cartridge.MapperNo == 9)
                {
                    if ((Address >= 0x1fd0) && (Address <= 0x1fdf))
                    {
                        Map9.latch1 = 0xfd;
                        Switch4kChrRom(Map9.latch1data1, 1);
                    }
                    else if ((Address >= 0x1fe0) && (Address <= 0x1fef))
                    {
                        Map9.latch1 = 0xfe;
                        Switch4kChrRom(Map9.latch1data2, 1);
                    }
                }
                else if (_Cartridge.MapperNo == 10)
                {
                    if ((Address >= 0x1fd0) && (Address <= 0x1fdf))
                    {
                        Map10.latch2 = 0xfd;
                        Switch4kChrRom(Map10.latch2data1, 1);
                    }
                    else if ((Address >= 0x1fe0) && (Address <= 0x1fef))
                    {
                        Map10.latch2 = 0xfe;
                        Switch4kChrRom(Map10.latch2data2, 1);
                    }
                }
                returnvalue = _Cartridge.CHR[current_chr_rom_page[7]][Address - 0x1C00];
            }
            return returnvalue;
        }
        public void TickScanlineTimer()
        { _CurrentMapper.TickScanlineTimer(); }
        public void TickCycleTimer()
        { _CurrentMapper.TickCycleTimer(); }
        //* 8
        public void Switch32kPrgRom(int start)
        {
            int i;
            switch (_Cartridge.PRG_PAGES)
            {
                case (2): start = (start & 0x7); break;
                case (4): start = (start & 0xf); break;
                case (8): start = (start & 0x1f); break;
                case (16): start = (start & 0x3f); break;
                case (32): start = (start & 0x7f); break;
                case (64): start = (start & 0xff); break;
                case (128): start = (start & 0x1ff); break;
            }
            for (i = 0; i < 8; i++)
            {
                current_prg_rom_page[i] = (uint)(start + i);
            }
        }
        //* 4
        //area 0,1
        public void Switch16kPrgRom(int start, int area)
        {
            int i;
            switch (_Cartridge.PRG_PAGES)
            {
                case (2): start = (start & 0x7); break;
                case (4): start = (start & 0xf); break;
                case (8): start = (start & 0x1f); break;
                case (16): start = (start & 0x3f); break;
                case (31): start = (start & 0x7f); break;
                case (32): start = (start & 0x7f); break;
                case (64): start = (start & 0xff); break;
                case (128): start = (start & 0x1ff); break;
            }
            for (i = 0; i < 4; i++)
            {
                current_prg_rom_page[4 * area + i] = (uint)(start + i);
            }
        }
        //* 2
        //area 0,1,2,3
        public void Switch8kPrgRom(int start, int area)
        {
            int i;
            switch (_Cartridge.PRG_PAGES)
            {
                case (2): start = (start & 0x7); break;
                case (4): start = (start & 0xf); break;
                case (8): start = (start & 0x1f); break;
                case (16): start = (start & 0x3f); break;
                case (32): start = (start & 0x7f); break;
                case (64): start = (start & 0xff); break;
                case (128): start = (start & 0x1ff); break;
            }
            for (i = 0; i < 2; i++)
            {
                current_prg_rom_page[2 * area + i] = (uint)(start + i);
            }
        }
        //* 8
        public void Switch8kChrRom(int start)
        {
            int i;
            switch (_Cartridge.CHR_PAGES)
            {
                case (2): start = (start & 0xf); break;
                case (4): start = (start & 0x1f); break;
                case (8): start = (start & 0x3f); break;
                case (16): start = (start & 0x7f); break;
                case (32): start = (start & 0xff); break;
                case (64): start = (start & 0x1ff); break;
            }
            for (i = 0; i < 8; i++)
            {
                current_chr_rom_page[i] = (uint)(start + i);
            }
        }
        //* 4
        //area 0,1
        public void Switch4kChrRom(int start, int area)
        {
            int i;
            switch (_Cartridge.CHR_PAGES)
            {
                case (2): start = (start & 0xf); break;
                case (4): start = (start & 0x1f); break;
                case (8): start = (start & 0x3f); break;
                case (16): start = (start & 0x7f); break;
                case (32): start = (start & 0xff); break;
                case (64): start = (start & 0x1ff); break;
            }
            for (i = 0; i < 4; i++)
            {
                current_chr_rom_page[4 * area + i] = (uint)(start + i);
            }
        }
        //* 2 
        //area 0,1,2,3
        public void Switch2kChrRom(int start, int area)
        {
            int i;
            switch (_Cartridge.CHR_PAGES)
            {
                case (2): start = (start & 0xf); break;
                case (4): start = (start & 0x1f); break;
                case (8): start = (start & 0x3f); break;
                case (16): start = (start & 0x7f); break;
                case (32): start = (start & 0xff); break;
                case (64): start = (start & 0x1ff); break;
            }
            for (i = 0; i < 2; i++)
            {
                current_chr_rom_page[2 * area + i] = (uint)(start + i);
            }
        }
        //area 0,1,2,3,4,5,6,7
        public void Switch1kChrRom(int start, int area)
        {
            switch (_Cartridge.CHR_PAGES)
            {
                case (2): start = (start & 0xf); break;
                case (4): start = (start & 0x1f); break;
                case (8): start = (start & 0x3f); break;
                case (16): start = (start & 0x7f); break;
                case (32): start = (start & 0xff); break;
                case (64): start = (start & 0x1ff); break;
            }
            current_chr_rom_page[area] = (uint)(start);
        }
        //area 0,1,2,3
        public void Switch1kVROMToVRAM(int start, int area)
        {
            switch (_Cartridge.CHR_PAGES)
            {
                case (2): start = (start & 0xf); break;
                case (4): start = (start & 0x1f); break;
                case (8): start = (start & 0x3f); break;
                case (16): start = (start & 0x7f); break;
                case (32): start = (start & 0xff); break;
                case (64): start = (start & 0x1ff); break;
            }
            //current_chr_rom_page[area] = (uint)(start);
            switch (area)
            {
                case 0:
                    for (int i = 0; i < 0x400; i++)
                    {
                        _Nes.PPU.VRAM[i] = _Cartridge.CHR[start][i];
                    }
                    break;
                case 1:
                    for (int i = 0x400; i < 0x800; i++)
                    {
                        _Nes.PPU.VRAM[i] = _Cartridge.CHR[start][i - 0x400];
                    }
                    break;
                case 2:
                    for (int i = 0x800; i < 0xC00; i++)
                    {
                        _Nes.PPU.VRAM[i] = _Cartridge.CHR[start][i - 0x800];
                    }
                    break;
                case 3:
                    for (int i = 0xC00; i < 0x1000; i++)
                    {
                        _Nes.PPU.VRAM[i] = _Cartridge.CHR[start][i - 0xC00];
                    }
                    break;
            }
        }
        //area 0,1,2,3
        public void SwitchVRAMToVRAM(int start, int area)
        {
            //current_chr_rom_page[area] = (uint)(start);
            switch (start)
            {
                case 0: start = 0x000; break;
                case 1: start = 0x400; break;
                case 2: start = 0x800; break;
                case 3: start = 0xC00; break;
            }
            switch (area)
            {
                case 0:
                    for (int i = 0; i < 0x400; i++)
                    {
                        _Nes.PPU.VRAM[i] = _Nes.PPU.VRAM[i + start];
                    }
                    break;
                case 1:
                    for (int i = 0x400; i < 0x800; i++)
                    {
                        _Nes.PPU.VRAM[i] = _Nes.PPU.VRAM[i - 0x400 + start];
                    }
                    break;
                case 2:
                    for (int i = 0x800; i < 0xC00; i++)
                    {
                        _Nes.PPU.VRAM[i] = _Nes.PPU.VRAM[i - 0x800 + start];
                    }
                    break;
                case 3:
                    for (int i = 0xC00; i < 0x1000; i++)
                    {
                        _Nes.PPU.VRAM[i] = _Nes.PPU.VRAM[i - 0xC00 + start];
                    }
                    break;
            }
        }
        public void InitializeMapper()
        {
            switch (_Cartridge.MapperNo)
            {
                case 0:
                    _CurrentMapper = new Mapper00(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 1:
                    _CurrentMapper = new Mapper01(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 2:
                    _CurrentMapper = new Mapper02(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 3:
                    _CurrentMapper = new Mapper03(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 4:
                    _CurrentMapper = new Mapper04(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 5:
                    _CurrentMapper = new Mapper05(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 6:
                    _CurrentMapper = new Mapper06(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 7:
                    _CurrentMapper = new Mapper07(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 8:
                    _CurrentMapper = new Mapper08(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 9:
                    _CurrentMapper = new Mapper09(this);
                    Map9 = (Mapper09)_CurrentMapper;
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 10:
                    _CurrentMapper = new Mapper10(this);
                    Map10 = (Mapper10)_CurrentMapper;
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 11:
                    _CurrentMapper = new Mapper11(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 13:
                    _CurrentMapper = new Mapper13(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 15:
                    _CurrentMapper = new Mapper15(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 16:
                    _CurrentMapper = new Mapper16(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 17:
                    _CurrentMapper = new Mapper17(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 18:
                    _CurrentMapper = new Mapper18(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 19:
                    _CurrentMapper = new Mapper19(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 21:
                    _CurrentMapper = new Mapper21(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 22:
                    _CurrentMapper = new Mapper22(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 23:
                    _CurrentMapper = new Mapper23(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 24:
                    _CurrentMapper = new Mapper24(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 32:
                    _CurrentMapper = new Mapper32(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 33:
                    _CurrentMapper = new Mapper33(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 34:
                    _CurrentMapper = new Mapper34(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 41:
                    _CurrentMapper = new Mapper41(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 48:
                    _CurrentMapper = new Mapper33(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 61:
                    _CurrentMapper = new Mapper61(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 64:
                    _CurrentMapper = new Mapper64(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 65:
                    _CurrentMapper = new Mapper65(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 66:
                    _CurrentMapper = new Mapper66(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 69:
                    _CurrentMapper = new Mapper69(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 71:
                    _CurrentMapper = new Mapper71(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 78:
                    _CurrentMapper = new Mapper71(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 79:
                    _CurrentMapper = new Mapper79(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 80:
                    _CurrentMapper = new Mapper80(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 81:
                    _CurrentMapper = new Mapper113(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 82:
                    _CurrentMapper = new Mapper82(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 91:
                    _CurrentMapper = new Mapper91(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 113:
                    _CurrentMapper = new Mapper113(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 225:
                    _CurrentMapper = new Mapper225_255(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
                case 255:
                    _CurrentMapper = new Mapper225_255(this);
                    _CurrentMapper.SetUpMapperDefaults();
                    break;
            }
        }

        //Properties
        /// <summary>
        /// Get the current cart
        /// </summary>
        public Cart Cartridge
        { get { return _Cartridge; } }
        public IMapper CurrentMapper
        { get { return _CurrentMapper; } set { _CurrentMapper = value; } }
        public bool IsSRAMReadOnly
        { get { return _IsSRAMReadOnly; } set { _IsSRAMReadOnly = value; } }
        public NesEngine NES
        { get { return _Nes; } }
    }
}
