﻿/*
This file is part of My Nes
A Nintendo Entertainment System Emulator.

 Copyright © 2009 - 2010 Ala Hadid (AHD)

My Nes is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

My Nes is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyNes.Nes
{
    class Mapper24 : IMapper
    {
        MAP Map;
        int irq_latch = 0;
        bool irq_enable = false;
        int irq_counter = 0;
        int irq_clock = 0;
        public Mapper24(MAP MAP)
        { Map = MAP; }
        public void Write(ushort address, byte data)
        {
            switch (address)
            {
                case 0x8000: Map.Switch16kPrgRom(data * 4, 0); break;
                case 0xB003:
                    switch ((data & 0xC) >> 2)
                    {
                        case 0: Map.Cartridge.Mirroring = MIRRORING.HORIZONTAL; break;
                        case 1: Map.Cartridge.Mirroring = MIRRORING.VERTICAL; break;
                        case 2:
                            Map.Cartridge.Mirroring = MIRRORING.ONE_SCREEN;
                            Map.Cartridge.MirroringBase = 0x2000;
                            break;
                        case 3:
                            Map.Cartridge.Mirroring = MIRRORING.ONE_SCREEN;
                            Map.Cartridge.MirroringBase = 0x2400;
                            break;
                    }
                    break;

                case 0xC000: Map.Switch8kPrgRom(data * 2, 2); break;
                case 0xD000: Map.Switch1kChrRom(data, 0); break;
                case 0xD001: Map.Switch1kChrRom(data, 1); break;
                case 0xD002: Map.Switch1kChrRom(data, 2); break;
                case 0xD003: Map.Switch1kChrRom(data, 3); break;
                case 0xE000: Map.Switch1kChrRom(data, 4); break;
                case 0xE001: Map.Switch1kChrRom(data, 5); break;
                case 0xE002: Map.Switch1kChrRom(data, 6); break;
                case 0xE003: Map.Switch1kChrRom(data, 7); break;

                case 0xF000:
                    irq_latch = data;
                    break;
                case 0xF001:
                    irq_enable = (data & 0x01) != 0;
                    break;
                case 0xF002:
                    irq_counter = irq_latch;
                    break;
            }
        }
        public void SetUpMapperDefaults()
        {
            Map.Switch16kPrgRom(0, 0);
            Map.Switch16kPrgRom((Map.Cartridge.PRG_PAGES - 1) * 4, 1);
            //Map.Switch8kChrRom(0);
            DEBUG.WriteLine(this, "Mapper 24 setup OK.", DebugStatus.Cool);
        }
        public void TickScanlineTimer()
        {

        }
        public void TickCycleTimer()
        {
            if (irq_enable)
            {
                if ((irq_clock += Map.NES.CPU.CycleCounter) >= 0x72)
                {
                    irq_clock -= 0x72;
                    if (irq_counter == 0xFF)
                    {
                        irq_counter = irq_latch;
                        Map.NES.CPU.IRQNextTime = true;
                    }
                    else
                    {
                        irq_counter++;
                    }
                }
            }
        }
        public void SoftReset()
        {

        }
        public bool WriteUnder8000
        {
            get { return true; }
        }
        public bool WriteUnder6000
        {
            get { return true; }
        }
    }
}
