﻿/*
This file is part of My Nes
A Nintendo Entertainment System Emulator.

 Copyright © 2009 - 2010 Ala Hadid (AHD)

My Nes is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

My Nes is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyNes.Nes
{
    class Mapper91 : IMapper
    {
        MAP Map;
        bool IRQEnabled;
        int IRQCount = 0;
        public Mapper91(MAP map)
        { Map = map; }
        public void Write(ushort address, byte data)
        {
            if (address == 0x6000)
                Map.Switch2kChrRom(data * 2, 0);
            else if (address == 0x6001)
                Map.Switch2kChrRom(data * 2, 1);
            else if (address == 0x6002)
                Map.Switch2kChrRom(data * 2, 2);
            else if (address == 0x6003)
                Map.Switch2kChrRom(data * 2, 3);
            else if (address == 0x7000)
                Map.Switch8kPrgRom(data * 2, 0);
            else if (address == 0x7001)
                Map.Switch8kPrgRom(data * 2, 1);
            else if (address == 0x7006)
            {
                IRQEnabled = false;
                IRQCount = 0;
            }
            else if (address == 0x7007)
                IRQEnabled = true;
        }
        public void SetUpMapperDefaults()
        {
            Map.Switch16kPrgRom((Map.Cartridge.PRG_PAGES - 1) * 4, 1);
            Map.Switch8kChrRom(0);
            DEBUG.WriteLine(this, "Mapper 91 setup OK.", DebugStatus.Cool);
        }
        public void TickScanlineTimer()
        {
            if (IRQCount < 8 & IRQEnabled)
            {
                IRQCount++;
                if (IRQCount >= 8)
                {
                    Map.NES.CPU.IRQNextTime = true;
                }
            }
        }
        public void TickCycleTimer()
        {

        }
        public void SoftReset()
        {

        }
        public bool WriteUnder8000
        {
            get { return true; }
        }
        public bool WriteUnder6000
        {
            get { return false; }
        }
    }
}
