﻿/*
This file is part of My Nes
A Nintendo Entertainment System Emulator.

 Copyright © 2009 - 2010 Ala Hadid (AHD)

My Nes is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

My Nes is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyNes.Nes.Input;

namespace MyNes.Nes
{
    /// <summary>
    /// The main class for memory including RAM, IO, SRAM and PRG
    /// </summary>
    public class Memory
    {
        /*Memory shared by CPU & PPU*/
        public Memory(NesEngine Engine)
        {
            _Nes = Engine;
            InitializeMemory();
        }
        byte[] _RAM;
        byte[] _SRAM;
        MAP _MAP;
        NesEngine _Nes;
        public int JoyData1 = 0;
        public int JoyData2 = 0;
        public byte JoyStrobe = 0;
        InputManager _inputManager;
        Joypad _joypad1;
        Joypad _joypad2;
        public Zapper ZAPPER;
        public bool WriteOnRam = true;
        #region Memory Read Write
        /// <summary>
        /// Write a value into the memory at the specific address
        /// </summary>
        /// <param name="Address">The address to write into</param>
        /// <param name="Value">The value to write</param>
        /// <returns></returns>
        public byte Write(ushort Address, byte Value)
        {
            if (Address < 0x2000)
            {
                if (WriteOnRam)
                    _RAM[Address & 0x07FF] = Value;
            }
            else if (Address < 0x4000)/*IO,0x2xxx*/
            {
                if (Address == 0x2000)
                    _Nes.PPU.Write2000(Value);
                else if (Address == 0x2001)
                    _Nes.PPU.Write2001(Value);
                else if (Address == 0x2003)
                    _Nes.PPU.Write2003(Value);
                else if (Address == 0x2004)
                    _Nes.PPU.Write2004(Value);
                else if (Address == 0x2005)
                    _Nes.PPU.Write2005(Value);
                else if (Address == 0x2006)
                    _Nes.PPU.Write2006(Value);
                else if (Address == 0x2007)
                    _Nes.PPU.Write2007(Value);
            }
            else if (Address < 0x6000)/*IO,0x4xxx*/
            {
                if (Address == 0x4000)
                    _Nes.APU.Write_4000(Value);
                else if (Address == 0x4001)
                    _Nes.APU.Write_4001(Value);
                else if (Address == 0x4002)
                    _Nes.APU.Write_4002(Value);
                else if (Address == 0x4003)
                    _Nes.APU.Write_4003(Value);
                else if (Address == 0x4004)
                    _Nes.APU.Write_4004(Value);
                else if (Address == 0x4005)
                    _Nes.APU.Write_4005(Value);
                else if (Address == 0x4006)
                    _Nes.APU.Write_4006(Value);
                else if (Address == 0x4007)
                    _Nes.APU.Write_4007(Value);
                else if (Address == 0x4008)
                    _Nes.APU.Write_4008(Value);
                else if (Address == 0x400A)
                    _Nes.APU.Write_400A(Value);
                else if (Address == 0x400B)
                    _Nes.APU.Write_400B(Value);
                else if (Address == 0x400C)
                    _Nes.APU.Write_400C(Value);
                else if (Address == 0x400E)
                    _Nes.APU.Write_400E(Value);
                else if (Address == 0x400F)
                    _Nes.APU.Write_400F(Value);
                else if (Address == 0x4010)
                    _Nes.APU.Write_4010(Value);
                else if (Address == 0x4011)
                    _Nes.APU.Write_4011(Value);
                else if (Address == 0x4012)
                    _Nes.APU.Write_4012(Value);
                else if (Address == 0x4013)
                    _Nes.APU.Write_4013(Value);
                else if (Address == 0x4014)
                    _Nes.PPU.Write4014(Value);
                else if (Address == 0x4015)
                    _Nes.APU.Write_4015(Value);
                else if (Address == 0x4016)
                {
                    if ((JoyStrobe == 1) && ((Value & 1) == 0))
                    {
                        _inputManager.Update();
                        JoyData1 = _joypad1.GetJoyData();
                        JoyData2 = _joypad2.GetJoyData();
                    }
                    JoyStrobe = (byte)(Value & 1);
                }
                else if (Address == 0x4017)
                    _Nes.APU.Write_4017(Value);

                if (_MAP.CurrentMapper.WriteUnder6000)
                    _MAP.WritePRG(Address, Value);
            }
            else if (Address < 0x8000)/*SRAM, Trainer*/
            {
                if (!_MAP.IsSRAMReadOnly)
                    _SRAM[Address - 0x6000] = Value;
                if (_MAP.CurrentMapper.WriteUnder8000)
                    _MAP.WritePRG(Address, Value);
            }
            else/*PRG*/
            {
                _MAP.WritePRG(Address, Value);
            }
            return 1;
        }
        /// <summary>
        /// Read a value from the memory at a specific address
        /// </summary>
        /// <param name="Address">The address to read from</param>
        /// <returns>Byte The value</returns>
        public byte Read(ushort Address)
        {
            if (Address < 0x2000)/*RAM*/
            {
                return _RAM[Address & 0x07FF];
            }
            else if (Address < 0x6000)/*IO*/
            {
                if (Address == 0x2002)
                    return _Nes.PPU.Read2002();
                else if (Address == 0x2004)
                    return _Nes.PPU.Read2004();
                else if (Address == 0x2007)
                    return _Nes.PPU.Read2007();
                else if (Address == 0x4015)
                    return _Nes.APU.Read_4015();
                else if (Address == 0x4016)
                {
                    byte v = (byte)(0x40 | (JoyData1 & 1));
                    JoyData1 = (JoyData1 >> 1) | 0x80;
                    return v;
                }
                else if (Address == 0x4017)
                {
                    byte v = (byte)(0x40 | ZAPPER.GetData() | (JoyData2 & 1));
                    JoyData2 = (JoyData2 >> 1) | 0x80;
                    return v;
                }
                else if (_MAP.Cartridge.MapperNo == 225 | _MAP.Cartridge.MapperNo == 255)
                {
                    Mapper225_255 map225 = (Mapper225_255)_MAP.CurrentMapper;
                    return map225.ReadRegisters(Address);
                }
            }
            else if (Address < 0x8000)/*SRAM*/
            {
                if (_MAP.Cartridge.MapperNo == 5)
                    return 1;
                return _SRAM[Address - 0x6000];
            }
            else/*PRG*/
            {
                return _MAP.ReadPRG(Address);
            }
            return 0;
        }
        /// <summary>
        /// Read an address from the memory
        /// </summary>
        /// <param name="Address"></param>
        /// <returns></returns>
        public ushort Read16(ushort Address)
        {
            byte data_1 = 0;
            byte data_2 = 0;
            if (Address < 0x2000)
            {
                data_1 = _RAM[(Address & 0x07FF)];
                data_2 = _RAM[(Address & 0x07FF) + 1];
            }
            else if (Address < 0x8000)
            {
                data_1 = _SRAM[Address - 0x6000];
                data_2 = _SRAM[(Address - 0x6000) + 1];
            }
            else
            {
                data_1 = _MAP.ReadPRG(Address);
                data_2 = _MAP.ReadPRG((ushort)(Address + 1));
            }
            return (ushort)((data_2 << 8) | data_1);
        }
        #endregion
        /// <summary>
        /// Initialize and clear the memory
        /// </summary>
        void InitializeMemory()
        {
            _RAM = new byte[0x800];
            _MAP = new MAP(this, _Nes);
            _SRAM = new byte[0x2000];
            ZAPPER = new Zapper(_Nes);
            DEBUG.WriteLine(this, "Memory initialized ok.", DebugStatus.Cool);
        }
        public bool LoadCart(string RomPath)
        {
            bool Done = _MAP.Cartridge.LoadCart(RomPath);
            if (Done)
                _MAP.InitializeMapper();
            return Done;
        }
        //Properties
        /// <summary>
        /// Get or set the RAM pages, addresses 0x0000 -> 0x2000
        /// </summary>
        public byte[] RAM
        { get { return _RAM; } set { _RAM = value; } }
        /// <summary>
        /// Get or set the Save Ram, addresses 0x6000 -> 0x8000
        /// </summary>
        public byte[] SRAM
        { get { return _SRAM; } set { _SRAM = value; } }
        public MAP MAP
        { get { return _MAP; } }
        public Joypad Joypad1
        { get { return _joypad1; } set { _joypad1 = value; } }
        public Joypad Joypad2
        { get { return _joypad2; } set { _joypad2 = value; } }
        public InputManager InputManager
        { get { return _inputManager; } set { _inputManager = value; } }
    }
}