﻿/*
This file is part of My Nes
A Nintendo Entertainment System Emulator.

 Copyright © 2009 - 2010 Ala Hadid (AHD)

My Nes is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

My Nes is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;

namespace MyNes.Nes
{
    public class State
    {
        NesEngine _Nes;
        StateHolder _StateHolder = new StateHolder();
        /// <summary>
        /// The state saver / loader
        /// </summary>
        /// <param name="NesEmu">The current system you want to save / load state from / into</param>
        public State(NesEngine NesEmu)
        {
            _Nes = NesEmu;
        }
        public bool SaveState(string FilePath)
        {
            try
            {
                _Nes.Pause();
                _StateHolder.LoadNesData(_Nes);
                FileStream fs = new FileStream(FilePath, FileMode.Create);
                BinaryFormatter formatter = new BinaryFormatter();
                formatter.Serialize(fs, _StateHolder);
                fs.Close();
                _Nes.Resume();
                return true;
            }
            catch { return false; }
        }
        public bool LoadState(string FilePath)
        {
            try
            {
                if (File.Exists(FilePath))
                {
                    _Nes.Pause();
                    FileStream fs = new FileStream(FilePath, FileMode.Open);
                    BinaryFormatter formatter = new BinaryFormatter();
                    _StateHolder = (StateHolder)formatter.Deserialize(fs);
                    fs.Close();
                    _StateHolder.ApplyDataToNes(_Nes);
                    _Nes.Resume();
                    return true;
                }
                return false;
            }
            catch { return false; };
        }
    }
    [Serializable()]
    class StateHolder
    {
        #region CPU
        byte REG_A = 0;
        byte REG_X = 0;
        byte REG_Y = 0;
        byte REG_S = 0;
        ushort REG_PC = 0;
        bool Flag_N = false;
        bool Flag_V = false;
        bool Flag_B = false;
        bool Flag_D = false;
        bool Flag_I = true;
        bool Flag_Z = false;
        bool Flag_C = false;
        public int CycleCounter = 0;
        int CyclesPerScanline = 0;
        byte OpCode = 0;
        ushort PrevPC = 0;
        #endregion
        #region MEMORY
        byte[] _RAM;
        byte[] _SRAM;
        int JoyData1 = 0;
        int JoyData2 = 0;
        byte JoyStrobe = 0;
        #endregion
        #region CART
        byte[][] _CHR;
        MIRRORING _MIRRORING;
        bool _save_ram_present;
        bool _is_vram;
        uint _mirroringBase;
        #endregion
        #region MAP
        public uint[] current_prg_rom_page;
        public uint[] current_chr_rom_page;
        #endregion
        #region PPU
        byte[] SPRRAM;
        byte[] VRAM;
        int CurrentScanLine = 0;
        ushort VRAMAddress = 0;
        bool Sprite0Hit = false;
        int SpriteCrossed = 0;
        int ScanlinesPerFrame = 0;
        int ScanlineOfVBLANK = 0;
        int _FPS = 0;
        bool VBLANK = false;
        byte VRAMReadBuffer = 0;
        bool _NoLimiter = false;
        /*2000*/
        bool ExecuteNMIonVBlank = false;
        bool SpriteSize = false;//true=8x16, false=8x8
        int PatternTableAddressBackground = 0;
        int PatternTableAddress8x8Sprites = 0;
        int VRAMAddressIncrement = 1;
        byte ReloadBits2000 = 0;
        /*2001*/
        ushort ColorEmphasis = 0;
        bool SpriteVisibility = false;
        bool BackgroundVisibility = false;
        bool SpriteClipping = false;
        bool BackgroundClipping = false;
        bool MonochromeMode = false;
        /*2003*/
        byte SpriteRamAddress = 0;
        /*2005,2006*/
        bool PPUToggle = false;
        ushort VRAMTemp = 0;
        /*Draw stuff*/
        byte HScroll = 0;
        int VScroll = 0;
        int VBits = 0;
        int TileY = 0;
        #endregion
        #region APU
        int _FrameCounter = 0;
        bool _PAL;
        bool _FrameIRQ;
        #endregion
        #region MAPPERS
        //MAPPER 1
        int mapper1_register8000BitPosition;
        int mapper1_registerA000BitPosition;
        int mapper1_registerC000BitPosition;
        int mapper1_registerE000BitPosition;
        int mapper1_register8000Value;
        int mapper1_registerA000Value;
        int mapper1_registerC000Value;
        int mapper1_registerE000Value;
        byte mapper1_mirroringFlag;
        byte mapper1_onePageMirroring;
        byte mapper1_prgSwitchingArea;
        byte mapper1_prgSwitchingSize;
        byte mapper1_vromSwitchingSize;
        //MAPPER 4
        int mapper4_commandNumber;
        int mapper4_prgAddressSelect;
        int mapper4_chrAddressSelect;
        bool mapper4_timer_irq_enabled;
        uint mapper4_timer_irq_count;
        uint mapper4_timer_irq_reload;
        //MAPPER 5
        byte mapper5_prgBankSize;
        byte mapper5_chrBankSize;
        int mapper5_scanlineSplit;
        bool mapper5_splitIrqEnabled;
        //MAPPER 9
        byte mapper9_latch1;
        byte mapper9_latch2;
        int mapper9_latch1data1;
        int mapper9_latch1data2;
        int mapper9_latch2data1;
        int mapper9_latch2data2;
        //MAPPER 10
        byte mapper10_latch1;
        byte mapper10_latch2;
        int mapper10_latch1data1;
        int mapper10_latch1data2;
        int mapper10_latch2data1;
        int mapper10_latch2data2;
        //MAPPER 16
        short timer_irq_counter_16 = 0;
        short timer_irq_Latch_16 = 0;
        bool timer_irq_enabled;
        //MAPPER 18
        short Mapper18_Timer = 0;
        short Mapper18_latch = 0;
        byte mapper18_control = 0;
        int Mapper18_IRQWidth = 0;
        bool Mapper18_timer_irq_enabled;
        byte[] Mapper18_x = new byte[22];
        //MAPPER 225
        byte Mapper225_reg0 = 0xF;
        byte Mapper225_reg1 = 0xF;
        byte Mapper225_reg2 = 0xF;
        byte Mapper225_reg3 = 0xF;
        //MAPPER 32
        int mapper32SwitchingMode = 0;
        //MAPPER 41
        byte Mapper41_CHR_Low = 0;
        byte Mapper41_CHR_High = 0;
        //MAPPER 64
        byte mapper64_commandNumber;
        byte mapper64_prgAddressSelect;
        byte mapper64_chrAddressSelect;
        #endregion
        public void LoadNesData(NesEngine _Nes)
        {
            #region CPU
            REG_A = _Nes.CPU.REG_A;
            REG_X = _Nes.CPU.REG_X;
            REG_Y = _Nes.CPU.REG_Y;
            REG_S = _Nes.CPU.REG_S;
            REG_PC = _Nes.CPU.REG_PC;
            Flag_N = _Nes.CPU.Flag_N;
            Flag_V = _Nes.CPU.Flag_V;
            Flag_B = _Nes.CPU.Flag_B;
            Flag_D = _Nes.CPU.Flag_D;
            Flag_I = _Nes.CPU.Flag_I;
            Flag_Z = _Nes.CPU.Flag_Z;
            Flag_C = _Nes.CPU.Flag_C;
            CycleCounter = _Nes.CPU.CycleCounter;
            CyclesPerScanline = _Nes.CPU.CyclesPerScanline;
            OpCode = _Nes.CPU.OpCode;
            PrevPC = _Nes.CPU.PrevPC;
            #endregion
            #region MEMORY
            _RAM = _Nes.MEMORY.RAM;
            _SRAM = _Nes.MEMORY.SRAM;
            JoyData1 = _Nes.MEMORY.JoyData1;
            JoyData2 = _Nes.MEMORY.JoyData2;
            JoyStrobe = _Nes.MEMORY.JoyStrobe;
            #endregion
            #region CART
            if (_Nes.MEMORY.MAP.Cartridge.CHR_PAGES == 0)
                _CHR = _Nes.MEMORY.MAP.Cartridge.CHR;
            _MIRRORING = _Nes.MEMORY.MAP.Cartridge.Mirroring;
            _save_ram_present = _Nes.MEMORY.MAP.Cartridge.IsSaveRam;
            _is_vram = _Nes.MEMORY.MAP.Cartridge.IsVRAM;
            _mirroringBase = _Nes.MEMORY.MAP.Cartridge.MirroringBase;
            #endregion
            #region MAP
            current_prg_rom_page = _Nes.MEMORY.MAP.current_prg_rom_page;
            current_chr_rom_page = _Nes.MEMORY.MAP.current_chr_rom_page;
            #endregion
            #region PPU
            SPRRAM = _Nes.PPU.SPRRAM;
            VRAM = _Nes.PPU.VRAM;
            CurrentScanLine = _Nes.PPU.CurrentScanLine;
            VRAMAddress = _Nes.PPU.VRAMAddress;
            Sprite0Hit = _Nes.PPU.Sprite0Hit;
            SpriteCrossed = _Nes.PPU.SpriteCrossed;
            ScanlinesPerFrame = _Nes.PPU.ScanlinesPerFrame;
            ScanlineOfVBLANK = _Nes.PPU.ScanlineOfVBLANK;
            _FPS = _Nes.PPU._FPS;
            VBLANK = _Nes.PPU.VBLANK;
            VRAMReadBuffer = _Nes.PPU.VRAMReadBuffer;
            _NoLimiter = _Nes.PPU._NoLimiter;
            /*2000*/
            ExecuteNMIonVBlank = _Nes.PPU.ExecuteNMIonVBlank;
            SpriteSize = _Nes.PPU.SpriteSize;
            PatternTableAddressBackground = _Nes.PPU.PatternTableAddressBackground;
            PatternTableAddress8x8Sprites = _Nes.PPU.PatternTableAddress8x8Sprites;
            VRAMAddressIncrement = _Nes.PPU.VRAMAddressIncrement;
            ReloadBits2000 = _Nes.PPU.ReloadBits2000;
            /*2001*/
            ColorEmphasis = _Nes.PPU.ColorEmphasis;
            SpriteVisibility = _Nes.PPU.SpriteVisibility;
            BackgroundVisibility = _Nes.PPU.BackgroundVisibility;
            SpriteClipping = _Nes.PPU.SpriteClipping;
            BackgroundClipping = _Nes.PPU.BackgroundClipping;
            MonochromeMode = _Nes.PPU.MonochromeMode;
            /*2003*/
            SpriteRamAddress = _Nes.PPU.SpriteRamAddress;
            /*2005,2006*/
            PPUToggle = _Nes.PPU.PPUToggle;
            VRAMTemp = _Nes.PPU.VRAMTemp;
            /*Draw stuff*/
            HScroll = _Nes.PPU.HScroll;
            VScroll = _Nes.PPU.VScroll;
            VBits = _Nes.PPU.VBits;
            TileY = _Nes.PPU.TileY;
            #endregion
            #region APU
            _FrameCounter = _Nes.APU._FrameCounter;
            _PAL = _Nes.APU._PAL;
            _FrameIRQ = _Nes.APU._FrameIRQ;
            #endregion
            #region Mappers
            //MAPPER 1
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 1)
            {
                Mapper01 map1 = (Mapper01)_Nes.MEMORY.MAP.CurrentMapper;
                mapper1_register8000BitPosition = map1.mapper1_register8000BitPosition;
                mapper1_registerA000BitPosition = map1.mapper1_registerA000BitPosition;
                mapper1_registerC000BitPosition = map1.mapper1_registerC000BitPosition;
                mapper1_registerE000BitPosition = map1.mapper1_registerE000BitPosition;
                mapper1_register8000Value = map1.mapper1_register8000Value;
                mapper1_registerA000Value = map1.mapper1_registerA000Value;
                mapper1_registerC000Value = map1.mapper1_registerC000Value;
                mapper1_registerE000Value = map1.mapper1_registerE000Value;
                mapper1_mirroringFlag = map1.mapper1_mirroringFlag;
                mapper1_onePageMirroring = map1.mapper1_onePageMirroring;
                mapper1_prgSwitchingArea = map1.mapper1_prgSwitchingArea;
                mapper1_prgSwitchingSize = map1.mapper1_prgSwitchingSize;
                mapper1_vromSwitchingSize = map1.mapper1_vromSwitchingSize;
            }
            //MAPPER 4
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 4)
            {
                Mapper04 map4 = (Mapper04)_Nes.MEMORY.MAP.CurrentMapper;
                mapper4_commandNumber = map4.mapper4_commandNumber;
                mapper4_prgAddressSelect = map4.mapper4_prgAddressSelect;
                mapper4_chrAddressSelect = map4.mapper4_chrAddressSelect;
                mapper4_timer_irq_enabled = map4.timer_irq_enabled;
                mapper4_timer_irq_count = map4.timer_irq_count;
                mapper4_timer_irq_reload = map4.timer_irq_reload;
            }
            //MAPPER 5
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 5)
            {
                Mapper05 map5 = (Mapper05)_Nes.MEMORY.MAP.CurrentMapper;
                mapper5_prgBankSize = map5.mapper5_prgBankSize;
                mapper5_chrBankSize = map5.mapper5_chrBankSize;
                mapper5_scanlineSplit = map5.mapper5_scanlineSplit;
                mapper5_splitIrqEnabled = map5.mapper5_splitIrqEnabled;
            }
            //MAPPER 9
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 9)
            {
                Mapper09 map9 = (Mapper09)_Nes.MEMORY.MAP.CurrentMapper;
                mapper9_latch1 = map9.latch1;
                mapper9_latch2 = map9.latch2;
                mapper9_latch1data1 = map9.latch1data1;
                mapper9_latch1data2 = map9.latch1data2;
                mapper9_latch2data1 = map9.latch2data1;
                mapper9_latch2data2 = map9.latch2data2;
            }
            //MAPPER 10
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 10)
            {
                Mapper10 map10 = (Mapper10)_Nes.MEMORY.MAP.CurrentMapper;
                mapper10_latch1 = map10.latch1;
                mapper10_latch2 = map10.latch2;
                mapper10_latch1data1 = map10.latch1data1;
                mapper10_latch1data2 = map10.latch1data2;
                mapper10_latch2data1 = map10.latch2data1;
                mapper10_latch2data2 = map10.latch2data2;
            }
            //MAPPER 16
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 16)
            {
                Mapper16 map16 = (Mapper16)_Nes.MEMORY.MAP.CurrentMapper;
                timer_irq_counter_16 = map16.timer_irq_counter_16;
                timer_irq_Latch_16 = map16.timer_irq_Latch_16;
                timer_irq_enabled = map16.timer_irq_enabled;
            }
            //MAPPER 18
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 18)
            {
                Mapper18 map18 = (Mapper18)_Nes.MEMORY.MAP.CurrentMapper;
                Mapper18_Timer = map18.Mapper18_Timer;
                Mapper18_latch = map18.Mapper18_latch;
                mapper18_control = map18.mapper18_control;
                Mapper18_IRQWidth = map18.Mapper18_IRQWidth;
                Mapper18_timer_irq_enabled = map18.timer_irq_enabled;
                Mapper18_x = map18.x;
            }
            //MAPPER 225
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 225)
            {
                Mapper225_255 map225 = (Mapper225_255)_Nes.MEMORY.MAP.CurrentMapper;
                Mapper225_reg0 = map225.Mapper225_reg0;
                Mapper225_reg1 = map225.Mapper225_reg1;
                Mapper225_reg2 = map225.Mapper225_reg2;
                Mapper225_reg3 = map225.Mapper225_reg3;
            }
            //MAPPER 32
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 32)
            {
                Mapper32 map32 = (Mapper32)_Nes.MEMORY.MAP.CurrentMapper;
                mapper32SwitchingMode = map32.mapper32SwitchingMode;
            }
            //MAPPER 41
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 41)
            {
                Mapper41 map41 = (Mapper41)_Nes.MEMORY.MAP.CurrentMapper;
                Mapper41_CHR_Low = map41.Mapper41_CHR_Low;
                Mapper41_CHR_High = map41.Mapper41_CHR_High;
            }
            //MAPPER 64
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 64)
            {
                Mapper64 map64 = (Mapper64)_Nes.MEMORY.MAP.CurrentMapper;
                mapper64_commandNumber = map64.mapper64_commandNumber;
                mapper64_prgAddressSelect = map64.mapper64_prgAddressSelect;
                mapper64_chrAddressSelect = map64.mapper64_chrAddressSelect;
            }
            #endregion
        }
        public void ApplyDataToNes(NesEngine _Nes)
        {
            #region CPU
            _Nes.CPU.REG_A = REG_A;
            _Nes.CPU.REG_X = REG_X;
            _Nes.CPU.REG_Y = REG_Y;
            _Nes.CPU.REG_S = REG_S;
            _Nes.CPU.REG_PC = REG_PC;
            _Nes.CPU.Flag_N = Flag_N;
            _Nes.CPU.Flag_V = Flag_V;
            _Nes.CPU.Flag_B = Flag_B;
            _Nes.CPU.Flag_D = Flag_D;
            _Nes.CPU.Flag_I = Flag_I;
            _Nes.CPU.Flag_Z = Flag_Z;
            _Nes.CPU.Flag_C = Flag_C;
            _Nes.CPU.CycleCounter = CycleCounter;
            _Nes.CPU.CyclesPerScanline = CyclesPerScanline;
            _Nes.CPU.OpCode = OpCode;
            _Nes.CPU.PrevPC = PrevPC;
            #endregion
            #region MEMORY
            _Nes.MEMORY.RAM = _RAM;
            _Nes.MEMORY.SRAM = _SRAM;
            _Nes.MEMORY.JoyData1 = JoyData1;
            _Nes.MEMORY.JoyData2 = JoyData2;
            _Nes.MEMORY.JoyStrobe = JoyStrobe;
            #endregion
            #region CART
            if (_Nes.MEMORY.MAP.Cartridge.CHR_PAGES == 0)
                _Nes.MEMORY.MAP.Cartridge.CHR = _CHR;
            _Nes.MEMORY.MAP.Cartridge.Mirroring = _MIRRORING;
            _Nes.MEMORY.MAP.Cartridge.IsSaveRam = _save_ram_present;
            _Nes.MEMORY.MAP.Cartridge.IsVRAM = _is_vram;
            _Nes.MEMORY.MAP.Cartridge.MirroringBase = _mirroringBase;
            #endregion
            #region MAP
            _Nes.MEMORY.MAP.current_prg_rom_page = current_prg_rom_page;
            _Nes.MEMORY.MAP.current_chr_rom_page = current_chr_rom_page;
            #endregion
            #region PPU
            _Nes.PPU.SPRRAM = SPRRAM;
            _Nes.PPU.VRAM = VRAM;
            _Nes.PPU.CurrentScanLine = CurrentScanLine;
            _Nes.PPU.VRAMAddress = VRAMAddress;
            _Nes.PPU.Sprite0Hit = Sprite0Hit;
            _Nes.PPU.SpriteCrossed = SpriteCrossed;
            _Nes.PPU.ScanlinesPerFrame = ScanlinesPerFrame;
            _Nes.PPU.ScanlineOfVBLANK = ScanlineOfVBLANK;
            _Nes.PPU._FPS = _FPS;
            _Nes.PPU.VBLANK = VBLANK;
            _Nes.PPU.VRAMReadBuffer = VRAMReadBuffer;
            _Nes.PPU._NoLimiter = _NoLimiter;
            /*2000*/
            _Nes.PPU.ExecuteNMIonVBlank = ExecuteNMIonVBlank;
            _Nes.PPU.SpriteSize = SpriteSize;
            _Nes.PPU.PatternTableAddressBackground = PatternTableAddressBackground;
            _Nes.PPU.PatternTableAddress8x8Sprites = PatternTableAddress8x8Sprites;
            _Nes.PPU.VRAMAddressIncrement = VRAMAddressIncrement;
            _Nes.PPU.ReloadBits2000 = ReloadBits2000;
            /*2001*/
            _Nes.PPU.ColorEmphasis = ColorEmphasis;
            _Nes.PPU.SpriteVisibility = SpriteVisibility;
            _Nes.PPU.BackgroundVisibility = BackgroundVisibility;
            _Nes.PPU.SpriteClipping = SpriteClipping;
            _Nes.PPU.BackgroundClipping = BackgroundClipping;
            _Nes.PPU.MonochromeMode = MonochromeMode;
            /*2003*/
            _Nes.PPU.SpriteRamAddress = SpriteRamAddress;
            /*2005,2006*/
            _Nes.PPU.PPUToggle = PPUToggle;
            _Nes.PPU.VRAMTemp = VRAMTemp;
            /*Draw stuff*/
            _Nes.PPU.HScroll = HScroll;
            _Nes.PPU.VScroll = VScroll;
            _Nes.PPU.VBits = VBits;
            _Nes.PPU.TileY = TileY;
            #endregion
            #region APU
            _Nes.APU._FrameCounter = _FrameCounter;
            _Nes.APU._PAL = _PAL;
            _Nes.APU._FrameIRQ = _FrameIRQ;
            #endregion
            #region MAPPERS
            //MAPPER 1
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo== 1)
            {
                Mapper01 map1 = (Mapper01)_Nes.MEMORY.MAP.CurrentMapper;
                map1.mapper1_register8000BitPosition = mapper1_register8000BitPosition;
                map1.mapper1_registerA000BitPosition = mapper1_registerA000BitPosition;
                map1.mapper1_registerC000BitPosition = mapper1_registerC000BitPosition;
                map1.mapper1_registerE000BitPosition = mapper1_registerE000BitPosition;
                map1.mapper1_register8000Value = mapper1_register8000Value;
                map1.mapper1_registerA000Value = mapper1_registerA000Value;
                map1.mapper1_registerC000Value = mapper1_registerC000Value;
                map1.mapper1_registerE000Value = mapper1_registerE000Value;
                map1.mapper1_mirroringFlag = mapper1_mirroringFlag;
                map1.mapper1_onePageMirroring = mapper1_onePageMirroring;
                map1.mapper1_prgSwitchingArea = mapper1_prgSwitchingArea;
                map1.mapper1_prgSwitchingSize = mapper1_prgSwitchingSize;
                map1.mapper1_vromSwitchingSize = mapper1_vromSwitchingSize;
            }
            //MAPPER 4
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 4)
            {
                Mapper04 map4 = (Mapper04)_Nes.MEMORY.MAP.CurrentMapper;
                map4.mapper4_commandNumber = mapper4_commandNumber;
                map4.mapper4_prgAddressSelect = mapper4_prgAddressSelect;
                map4.mapper4_chrAddressSelect = mapper4_chrAddressSelect;
                map4.timer_irq_enabled = mapper4_timer_irq_enabled;
                map4.timer_irq_count = mapper4_timer_irq_count;
                map4.timer_irq_reload = mapper4_timer_irq_reload;
            }
            //MAPPER 5
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 5)
            {
                Mapper05 map5 = (Mapper05)_Nes.MEMORY.MAP.CurrentMapper;
                map5.mapper5_prgBankSize = mapper5_prgBankSize;
                map5.mapper5_chrBankSize = mapper5_chrBankSize;
                map5.mapper5_scanlineSplit = mapper5_scanlineSplit;
                map5.mapper5_splitIrqEnabled = mapper5_splitIrqEnabled;
            }
            //MAPPER 9
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 9)
            {
                Mapper09 map9 = (Mapper09)_Nes.MEMORY.MAP.CurrentMapper;
                map9.latch1 = mapper9_latch1;
                map9.latch2 = mapper9_latch2;
                map9.latch1data1 = mapper9_latch1data1;
                map9.latch1data2 = mapper9_latch1data2;
                map9.latch2data1 = mapper9_latch2data1;
                map9.latch2data2 = mapper9_latch2data2;
            }
            //MAPPER 10
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 10)
            {
                Mapper10 map10 = (Mapper10)_Nes.MEMORY.MAP.CurrentMapper;
                map10.latch1 = mapper10_latch1;
                map10.latch2 = mapper10_latch2;
                map10.latch1data1 = mapper10_latch1data1;
                map10.latch1data2 = mapper10_latch1data2;
                map10.latch2data1 = mapper10_latch2data1;
                map10.latch2data2 = mapper10_latch2data2;
            }
            //MAPPER 16
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 16)
            {
                Mapper16 map16 = (Mapper16)_Nes.MEMORY.MAP.CurrentMapper;
                map16.timer_irq_counter_16 = timer_irq_counter_16;
                map16.timer_irq_Latch_16 = timer_irq_Latch_16;
                map16.timer_irq_enabled = timer_irq_enabled;
            }
            //MAPPER 18
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 18)
            {
                Mapper18 map18 = (Mapper18)_Nes.MEMORY.MAP.CurrentMapper;
                map18.Mapper18_Timer = Mapper18_Timer;
                map18.Mapper18_latch = Mapper18_latch;
                map18.mapper18_control = mapper18_control;
                map18.Mapper18_IRQWidth = Mapper18_IRQWidth;
                map18.timer_irq_enabled = Mapper18_timer_irq_enabled;
                map18.x = Mapper18_x;
            }
            //MAPPER 225
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 225)
            {
                Mapper225_255 map225 = (Mapper225_255)_Nes.MEMORY.MAP.CurrentMapper;
                map225.Mapper225_reg0 = Mapper225_reg0;
                map225.Mapper225_reg1 = Mapper225_reg1;
                map225.Mapper225_reg2 = Mapper225_reg2;
                map225.Mapper225_reg3 = Mapper225_reg3;
            }
            //MAPPER 32
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 32)
            {
                Mapper32 map32 = (Mapper32)_Nes.MEMORY.MAP.CurrentMapper;
                map32.mapper32SwitchingMode = mapper32SwitchingMode;
            }
            //MAPPER 41
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 41)
            {
                Mapper41 map41 = (Mapper41)_Nes.MEMORY.MAP.CurrentMapper;
                map41.Mapper41_CHR_Low = Mapper41_CHR_Low;
                map41.Mapper41_CHR_High = Mapper41_CHR_High;
            }
            //MAPPER 64
            if (_Nes.MEMORY.MAP.Cartridge.MapperNo == 64)
            {
                Mapper64 map64 = (Mapper64)_Nes.MEMORY.MAP.CurrentMapper;
                map64.mapper64_commandNumber = mapper64_commandNumber;
                map64.mapper64_prgAddressSelect = mapper64_prgAddressSelect;
                map64.mapper64_chrAddressSelect = mapper64_chrAddressSelect;
            }
            #endregion
        }
    }
}
